/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/*
 * This file contains the functions that launch browsers for e.g.
 * the language reference pages.
 *
 * A utility function browse_OpenUrlInDefaultBrowser(url)
 * is also provided, which can be used from elsewhere in Komodo.
 */

// open the given url or complain appropriately
function browse_OpenUrlInDefaultBrowser(url, browser) {
    var koWebbrowser = Components.classes['@activestate.com/koWebbrowser;1'].
                       getService(Components.interfaces.koIWebbrowser);
    var prefs = windowManager_getMainWindow().gPrefs;

    if (typeof(browser) == 'undefined' || !browser) {
        browser = prefs.getStringPref('browser');
    }
    var infoService = Components.classes['@activestate.com/koInfoService;1'].
                      getService(Components.interfaces.koIInfoService);
    var platform = infoService.platform;
    if (!platform.match(/^win/) && !platform.match(/^darwin/)
        && browser == '')
    {
        // Don't guess, since launching e.g. Mozilla can have side effects
        var browsers = koWebbrowser.get_possible_browsers(new Object());
        var answer = dialog_selectFromList(
                "Choose Default Browser",
                "Select the browser that Komodo should use to open URLs:",
                browsers,
                "one"); // selectionCondition
        if (answer == null) {
            return;
        }
        prefs.setStringPref("browser", answer[0]);
        browser = answer[0];
    }
    var ret = false;
    try {
        if (browser) {
            ret = koWebbrowser.open_new_browser(url,browser);
        } else {
            ret = koWebbrowser.open_new(url);
        }
    } catch (e) {
        log.exception(e);
    }
    if (!ret) {
        if (browser) {
            dialog_alert("Could not open the browser at "+browser+".");
        } else {
            dialog_alert("Could not file a graphical browser on the "+
                         "PATH with which to open '" + url + "'.");
        }
    }
}

function browse_ShowKeybindings()
{
    try {
        var text = gKeybindingMgr.makeCurrentKeyBindingTable();
        var tmpFileSvc = Components.classes["@activestate.com/koFileService;1"]
                         .getService(Components.interfaces.koIFileService)
        var outputTmpFile = tmpFileSvc.makeTempFile(".html","w");
        //dump('outputTmpFile name is '+outputTmpFile.URI+'\n');
        outputTmpFile.puts(text);
        outputTmpFile.flush();
        outputTmpFile.close();
        browse_OpenUrlInDefaultBrowser(outputTmpFile.URI);
    } catch(e) {
        log.exception(e);
    }
}

function browse_ShowCommandIds()
{
    try {
        var text = gKeybindingMgr.makeCommandIdTable();
        var tmpFileSvc = Components.classes["@activestate.com/koFileService;1"]
                         .getService(Components.interfaces.koIFileService)
        var outputTmpFile = tmpFileSvc.makeTempFile(".html","w");
        //dump('outputTmpFile name is '+outputTmpFile.URI+'\n');
        outputTmpFile.puts(text);
        outputTmpFile.flush();
        outputTmpFile.close();
        browse_OpenUrlInDefaultBrowser(outputTmpFile.URI);
    } catch(e) {
        log.exception(e);
    }
}

function browse_PerlRef_Local() {
    var perlInfoSvc = Components.classes["@activestate.com/koPerlInfoEx;1"]
                      .getService(Components.interfaces.koIAppInfoEx);
    browse_OpenUrlInDefaultBrowser(perlInfoSvc.localHelpFile);
}

function browse_PerlRef_Web() {
    var perlInfoSvc = Components.classes["@activestate.com/koPerlInfoEx;1"]
                      .getService(Components.interfaces.koIAppInfoEx);
    browse_OpenUrlInDefaultBrowser(perlInfoSvc.webHelpURL);
}

function browse_PerlMailingLists() {
    var url = "http://aspn.activestate.com/ASPN/Mail?topic=Perl";
    browse_OpenUrlInDefaultBrowser(url);
}

function browse_PythonRef_Local() {
    var pythonInfoSvc = Components.classes["@activestate.com/koPythonInfoEx;1"]
                      .getService(Components.interfaces.koIAppInfoEx);
    browse_OpenUrlInDefaultBrowser(pythonInfoSvc.localHelpFile);
}

function browse_PythonRef_Web() {
    var pythonInfoSvc = Components.classes["@activestate.com/koPythonInfoEx;1"]
                        .getService(Components.interfaces.koIAppInfoEx);
    browse_OpenUrlInDefaultBrowser(pythonInfoSvc.webHelpURL);
}

function browse_PythonMailingLists() {
    var url = "http://aspn.activestate.com/ASPN/Mail?topic=Python";
    browse_OpenUrlInDefaultBrowser(url);
}

function browse_PHPRef_Web() {
    var phpInfoSvc = Components.classes["@activestate.com/koPHPInfoEx;1"]
                     .getService(Components.interfaces.koIPHPInfoEx);
    browse_OpenUrlInDefaultBrowser(phpInfoSvc.webHelpURL);
}

function browse_PHPMailingLists() {
    var url = "http://aspn.activestate.com/ASPN/Mail?topic=PHP";
    browse_OpenUrlInDefaultBrowser(url);
}

function browse_RubyRef_Web() {
    var infoSvc = Components.classes["@activestate.com/koRubyInfoEx;1"]
                     .getService(Components.interfaces.koIAppInfoEx);
    browse_OpenUrlInDefaultBrowser(infoSvc.webHelpURL);
}

function browse_RubyMailingLists() {
    var url = "http://aspn.activestate.com/ASPN/Mail?topic=Ruby";
    browse_OpenUrlInDefaultBrowser(url);
}

function browse_TclRef_Local() {
    var tclInfoSvc = Components.classes["@activestate.com/koTclInfoEx;1"]
                      .getService(Components.interfaces.koIAppInfoEx);
    browse_OpenUrlInDefaultBrowser(tclInfoSvc.localHelpFile);
}

function browse_TclRef_Web() {
    var tclInfoSvc = Components.classes["@activestate.com/koTclInfoEx;1"]
                     .getService(Components.interfaces.koIAppInfoEx);
    browse_OpenUrlInDefaultBrowser(tclInfoSvc.webHelpURL);
}

function browse_TclMailingLists() {
    var url = "http://aspn.activestate.com/ASPN/Mail?topic=Tcl";
    browse_OpenUrlInDefaultBrowser(url);
}

function browse_XSLTMailingLists() {
    var url = "http://aspn.activestate.com/ASPN/Mail?topic=XSLT";
    browse_OpenUrlInDefaultBrowser(url);
}


function browse_UpdateHelpLanguagesPopup() {
    // Hide or show the local help entries in the Help->Languages popup
    // depending on whether an actually help file to launch can be found.
    var perlInfoSvc = Components.classes['@activestate.com/koPerlInfoEx;1'].
                      getService(Components.interfaces.koIAppInfoEx);
    var perlWidget = document.getElementById("menu_helpPerlRef_Local");
    var perlHelpFile = perlInfoSvc.localHelpFile;
    if (perlHelpFile) {
        perlWidget.removeAttribute("collapsed");
    } else {
        perlWidget.setAttribute("collapsed", true);
    }

    var pythonInfoSvc = Components.classes['@activestate.com/koPythonInfoEx;1'].
                        getService(Components.interfaces.koIAppInfoEx);
    var pythonWidget = document.getElementById("menu_helpPythonRef_Local");
    var pythonHelpFile = pythonInfoSvc.localHelpFile;
    if (pythonHelpFile) {
        pythonWidget.removeAttribute("collapsed");
    } else {
        pythonWidget.setAttribute("collapsed", true);
    }

    var tclInfoSvc = Components.classes['@activestate.com/koTclInfoEx;1'].
                     getService(Components.interfaces.koIAppInfoEx);
    var tclWidget = document.getElementById("menu_helpTclRef_Local");
    var tclHelpFile = tclInfoSvc.localHelpFile;
    if (tclHelpFile) {
        tclWidget.removeAttribute("collapsed");
    } else {
        tclWidget.setAttribute("collapsed", true);
    }
}


function browse_About() {
  window.open("chrome://jaguar/content/about.xul",
              "komodo_about",
              "chrome,dialog,modal");
}

function browse_ActiveState() {
    browse_OpenUrlInDefaultBrowser("http://www.ActiveState.com/")
}

function browse_ASPN() {
    browse_OpenUrlInDefaultBrowser("http://aspn.ActiveState.com/aspn/")
}

function browse_Support() {
    browse_OpenUrlInDefaultBrowser("http://www.ActiveState.com/Support/index.html")
}

function browse_Updates() {
    browse_OpenUrlInDefaultBrowser("http://aspn.ActiveState.com/ASPN/Downloads/")
}

function browse_Bugs()
{
    browse_OpenUrlInDefaultBrowser("http://bugs.ActiveState.com/Komodo/")
}
