/* Copyright (c) 2004-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* Object Browser top-level window */

// Possible TODOs:

//- Would be nice to be able to limit the search to a set of files, files
//  in a project, etc. This is general kind of "search scope" thing that
//  we need to handle/provide/use in other places as well: Code Browser,
//  find in files, etc.

//- Probably want link to Help... and a Close item. Would a <menu> fit in
//  here?

//- "File Path" icon needs the same styling that the "Code" tab tree uses
//  to provide some whitespace separation btwn icon and text.

//- On first start the "File" and "Language" strings on top of the scimoz
//  look weird. (BTW: if you don't have a name for this area, let's call
//  it the "Preview" area.) In fact, that area looks weird all the time.
//  That bar needs some kind of visual separation from the splitter
//  vertical area.

//  Or could possibly have a statusbar.

//- When a search finds nothing it would probably be helpful to put up a
//  dialog_alert() saying so. Otherwise a hitting "Search" can look like
//  it does nothing. Could have a "don't ask me again" if someone finds
//  that annoying.

//- The "double-click" to jump to main Komodo window was a little
//  surprising at first because I "lost" my "Symbol Finder" window when it
//  was pushed behind the main Komodo window. Don't know what to do about
//  that.

//- Should be a context menu on the Matches tree with some of the same
//  entries as the Code Browser context menu.

//- Did you consider adding a "Description" area that is the equivalent
//  of the Code Browser's description area?

// In module browse mode, it would be nice to right click on an import
// and be able to open it or add it to the list.

var log = getLoggingMgr().getLogger("objectbrowser");
//log.setLevel(LOG_DEBUG);

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var gPrefs = null;  // access to global prefs
var gWidgets = null; // cache of widget DOM nodes
var gTreeView = null;
var gBoxObject = null;
var os = Components.classes["@activestate.com/koOs;1"].getService();
var _gSortField = 'name';
var gCodeIntelSvc;
var _gInitDone = false;

function _initDialog() {
        if (_gInitDone) return;
        
    try {
        scintillaOverlayOnLoad();
        trace_onload();

        gPrefs = Components.classes["@activestate.com/koPrefService;1"].
                 getService(Components.interfaces.koIPrefService).prefs;
        gCodeIntelSvc = Components.classes["@activestate.com/koCodeIntelService;1"].
                 getService(Components.interfaces.koICodeIntelService);

        gWidgets = new Object();
        gWidgets.window = document.getElementById("komodo_objectbrowser");
        gWidgets.symbol = document.getElementById("symbol-textbox");
        gWidgets.module = document.getElementById("module-textbox");
        gWidgets.searchButton = document.getElementById("search-button");
        gWidgets.searchLabel = document.getElementById("search-label");
        gWidgets.exactMatch = document.getElementById("exact-match-checkbox");
        gWidgets.showImports = document.getElementById("show-imports-checkbox");
        gWidgets.tree = document.getElementById("results-tree");
        gWidgets.matchesLabel = document.getElementById("matches-label");
        gWidgets.previewDeck = document.getElementById("preview-deck");

        gWidgets.bufferView = document.getElementById("buffer-view");
        gWidgets.bufferView.initWithBuffer('', 'Text');
        gWidgets.fileLabel = document.getElementById("file-label");
        gTreeView = Components.classes["@activestate.com/koCodeIntelObjectBrowserTreeView;1"]
                              .createInstance();
        gWidgets.languagesMenupopup = document.getElementById('languages-menupopup');
        gWidgets.languagesMenulist = document.getElementById('languages-menulist');
        gBoxObject = gWidgets.tree.treeBoxObject;
        gBoxObject.view = gTreeView;

        _buildLanguagesMenu();
        _gInitDone = true;
    } catch (e) {
        log.exception(e);
    }
}

function OnLoad() {
    log.debug("OnLoad()");
    try {
        _initDialog();
        
        // Apply dialog initialization arguments.
        try {
            gWidgets.languagesMenulist.value = opener.gObjectBrowserLanguage;
        } catch (e) {
            log.exception(e);
        }
        if (opener.gObjectBrowserSymbol) {
            gWidgets.symbol.value = opener.gObjectBrowserSymbol;
        }
        if (opener.gObjectBrowserModule) {
            gWidgets.module.value = opener.gObjectBrowserModule;
        }
        gWidgets.exactMatch.checked = opener.gObjectBrowserExact;
        if (typeof(opener.gObjectBrowserShowImports) == 'undefined'
            || opener.gObjectBrowserShowImports == null) {
            gWidgets.showImports.checked = true;
        } else {
            gWidgets.showImports.checked = false;
        }

        UpdateSearch();
        _updateSortIndicators(_gSortField);
        gWidgets.symbol.focus();

        // Automatically search if a search symbol and/or module was given.
        if (gWidgets.symbol.value || gWidgets.module.value) {
            Search();
        }
        
        // Make sure the Object Browser window is brought to the front.
        // If the window was already up then calling back into it (via
        // "Find Symbol" in the main Komodo window) would not bring this
        // window to the front -- at least on MSWin32.
        window.focus();
    } catch (e) {
        log.exception(e);
    }
}
window.Reload = OnLoad;

function OnUnload() {

}

function _buildLanguagesMenu() {
    try {
        var menuitem;
        var languages = new Object();
        gCodeIntelSvc.getSupportedLanguages(new Object(), languages);
        var count;
        languages = languages.value;
        for (var i = 0; i < languages.length; i++)  {
            count = gTreeView.getFileCountForLanguage(languages[i]);
            menuitem = document.createElement('menuitem');
            menuitem.setAttribute('value', languages[i]);
            menuitem.setAttribute('label', "All "+ languages[i] + " files (" + count + " known)");
            if (count == 0) {
                menuitem.setAttribute('disabled', 'true');
            }
            gWidgets.languagesMenupopup.appendChild(menuitem);
        }
        count = gTreeView.getFileCountForLanguage('');
        menuitem = document.createElement('menuitem');
        menuitem.setAttribute('value', '');
        menuitem.setAttribute('label', "All files (" + count + " known)");
        gWidgets.languagesMenupopup.appendChild(menuitem);
    } catch (e) {
        log.exception(e);
    }
}

function UpdateSearch() {
    try {
        if (gWidgets.symbol.value == '' && gWidgets.module.value == '') {
            gWidgets.searchButton.setAttribute('disabled', 'true');
        } else {
            gWidgets.searchButton.removeAttribute('disabled');
        }
        if (gWidgets.symbol.value == '') {
            gWidgets.exactMatch.setAttribute('disabled', 'true');
            gWidgets.showImports.setAttribute('disabled', 'true');
            if (gWidgets.module.value) {
               gWidgets.searchLabel.setAttribute('value', "Browse modules matching '" + gWidgets.module.value + "'");
            } else {
               gWidgets.searchLabel.setAttribute('value', "Specify symbol and module search criteria");
            }
        } else {
            gWidgets.exactMatch.removeAttribute('disabled');
            gWidgets.showImports.removeAttribute('disabled');
            var label = "Search symbols";
            if (gWidgets.showImports.checked) {
                label += " (including imports)"
            }
            if (gWidgets.exactMatch.checked) {
                label += " named '" + gWidgets.symbol.value + "'";
            } else {
                label += " matching '" + gWidgets.symbol.value + "'";
            }
            if (gWidgets.module.value) {
                label += " in";
                if (gWidgets.languagesMenulist.value) {
                    label += " " + gWidgets.languagesMenulist.value;
                }
                label += " modules matching '" + gWidgets.module.value + "'";
            } else {
                if (gWidgets.languagesMenulist.value) {
                    label += " in " + gWidgets.languagesMenulist.value + " modules";
                }
            }
            gWidgets.searchLabel.setAttribute('value', label);
        }
    } catch (e) {
        log.exception(e);
    }
}
function Search() {
    try {
        if (gWidgets.searchButton.getAttribute('disabled') == 'true') {
            return;
        }
        var collectionId = '' ; // gWidgets.collectionList.selectedItem.id;
        var symbol = gWidgets.symbol.value;
        var module = gWidgets.module.value;
        var language = gWidgets.languagesMenulist.value;
        var exact = gWidgets.exactMatch.checked;
        var showImports = gWidgets.showImports.checked;

        var toggleIds = ['splitter1', 'splitter2', 'splitter3', 'splitter4',
                         'langType', 'module'];
        if (gWidgets.symbol.value == '') {
            for (id in toggleIds) {
                document.getElementById(toggleIds[id]).setAttribute('collapsed', 'true');
            }
        } else {
            for (id in toggleIds) {
                document.getElementById(toggleIds[id]).removeAttribute('collapsed');
            }
        }

        if (gWidgets.symbol.value) {
            MRU_addFromACTextbox(gWidgets.symbol);
        }
        if (gWidgets.module.value) {
            MRU_addFromACTextbox(gWidgets.module);
        }

        window.setCursor("wait");
        var start = new Date().getTime();
        gTreeView.search(collectionId, symbol, module, language,
                         exact, showImports);
        var end = new Date().getTime();
        var delta = ((end-start)/1000.0).toString();
        window.setCursor("auto");
        var numHits = gTreeView.rowCount;
        _updateSortIndicators(_gSortField);
        if (numHits != 0) {
            gWidgets.matchesLabel.value = String(numHits) + ' Matches (in ' + delta + ' seconds)';
            gTreeView.selection.select(0);
        } else {
            gWidgets.matchesLabel.value = 'No match found.';
        }
        SelectionChanged();
    } catch (e) {
        log.exception(e);
    }
}

function JumpToSelectedLine() {
    try {
        var path = gTreeView.currentNodeFilePath;
        var mainWin = windowManager_getMainWindow();
        if (!path) return;
        // don't do anything on .cix files
        if (path.indexOf('.cix') != -1) return;
        var line = gTreeView.currentNodeLine;
        // Sanity check that the file exists
        if (path.indexOf('koremote') == 0) { // assume remote files ok
            exists = true;
        } else if (path.indexOf('<Unsaved>') == 0) {
            // Need to check whether file still around
            var viewName = path.split(os.sep)[1]
            var view = mainWin.gViewMgr.getUntitledView(viewName);
            if (view) {
                view.makeCurrent();
                mainWin.focus();
                view.currentLine = line;
                return;
            } else {
                dialog_alert("The unsaved file '" + viewName + "' does not exist anymore (it was either saved or closed).");
                return;
            }
        } else {
            exists = os.path.exists(path);
        }
        if (!exists) {
            dialog_alert("The file '" + path + "' does not exist.");
            return;
        }
        if (line == -1) {
            dialog_alert("The selected symbol is defined in a binary file.");
        } else {
            mainWin = windowManager_getMainWindow();
            mainWin.focus();
            mainWin.gViewMgr.doFileOpenAtLine(path, line);
        }
    } catch (e) {
        log.exception(e);
    }
}

function SearchKeyPress(event) {
    try {
        if (event.keyCode == event.DOM_VK_ENTER ||
            event.keyCode == event.DOM_VK_RETURN) {
            event.stopPropagation();
            event.cancelBubble = true;
            event.preventDefault();
            Search();
        }
        UpdateSearch();
    } catch (e) {
        log.exception(e);
    }
}

function TreeOnClick(event) {
    try {
        // c.f. mozilla/mailnews/base/resources/content/threadPane.js
        var t = event.originalTarget;

        if (t.localName == "treecol") { // single-click on a column
            _columnClick(t.id);
        } else if (event.detail == 2 && t.localName == "treechildren") {
            // double-click in the tree body
            JumpToSelectedLine();
        }
    } catch (e) {
        log.exception(e);
    }
}


function _columnClick (columnID)
{
    gTreeView.sortBy(columnID);
    _updateSortIndicators(columnID);
}

function _updateSortIndicators (sortId) {
    var sortedColumn = null;

    // set the sort indicator on the column we are sorted by
    if (sortId) {
        sortedColumn = document.getElementById(sortId);
        if (sortedColumn) {
            var sortDirection = sortedColumn.getAttribute("sortDirection");
            if (sortDirection && sortDirection == "ascending") {
                sortedColumn.setAttribute("sortDirection", "descending");
            } else {
                sortedColumn.setAttribute("sortDirection", "ascending");
            }
        }
    }

    // remove the sort indicator from all the columns
    // except the one we are sorted by
    var currCol = gWidgets.tree.firstChild.firstChild;
    while (currCol) {
        while (currCol && currCol.localName != "treecol") {
            currCol = currCol.nextSibling;
        }
        if (currCol && (currCol != sortedColumn)) {
            currCol.removeAttribute("sortDirection");
        }
        if (currCol) {
            currCol = currCol.nextSibling;
        }
    }
}


function BufferKeyPress(event) {
    try {
        if (event.keyCode == event.DOM_VK_TAB) {
            if (event.shiftKey) {
                gWidgets.tree.focus();
            } else {
                gWidgets.symbol.focus();
            }
            event.stopPropagation();
            event.cancelBubble = true;
            event.preventDefault();
        }
    } catch (e) {
        log.exception(e);
    }

}

function TreeKeyPress(event) {
    try {
        if (event.keyCode == event.DOM_VK_ENTER ||
            event.keyCode == event.DOM_VK_RETURN) {
            event.stopPropagation();
            event.cancelBubble = true;
            event.preventDefault();
            JumpToSelectedLine();
        }
    } catch (e) {
        log.exception(e);
    }
}

function SelectionChanged() {
    try {
        var path = gTreeView.currentNodeFilePath;
        if (!path) {
            gWidgets.bufferView.setBufferText('');
            gWidgets.fileLabel.value = 'No Current File';
            return;
        }
        var language = gTreeView.currentNodeLanguage;
        var line = gTreeView.currentNodeLine-1;
        var buffer;
        var readable = os.path.exists(path);
        if (readable) {
            try {
                // XXX: Should deal with encodings, untitled documents,
                // remote files, etc.
                buffer = os.readfile(path);
            } catch (e) {
                readable = false;
            }
        }
        if (!readable) {
            gWidgets.previewDeck.selectedIndex = 1;
        } else {
            gWidgets.previewDeck.selectedIndex = 0;
            gWidgets.bufferView.setBufferText(buffer);
            gWidgets.bufferView.language = language;
        }
        gWidgets.fileLabel.value = path + ' (' + language + ')';
        var scimoz = gWidgets.bufferView.scimoz;
        var start = scimoz.positionFromLine(line);
        //var end = scimoz.getLineEndPosition(line);
        scimoz.anchor = scimoz.currentPos = start;
        scimoz.readOnly = true;

        // The following makes it easier to see the current line, but is
        // confusing.

        //scimoz.focus = true;
        //scin.caretPeriod = 0;
        //scimoz.anchor = end;
        fvl = scimoz.firstVisibleLine
        curLineNo = scimoz.lineFromPosition(scimoz.currentPos)
        scimoz.lineScroll(0, curLineNo-fvl-3)
        scimoz.currentPos = scimoz.positionFromLine(curLineNo);
        scimoz.lineEndExtend();
    } catch (e) {
        log.exception(e);
    }
}


