/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

// Note that we can never use the "current file's extension" or other
// simple rules to determine "can we debug".  *ANY* file may have a
// valid invocation setup.  So for now, this code assumes that
// ALL files are debuggable - this will mean that ALL files prompt
// for an invocation, which the user may cancel if they made
// an error.

function debugger_updateCommandset(commandset) {
    command_updateCommandset(commandset)
}

// Execute a "start debugger" command -
// the "Control" key switches whether to show the dialog
// or not, with the default being set by the dbgShowDialog pref
function debugger_doCommand(command, event)
{
    try {
        if (typeof(event) != 'undefined' && event)
            gDBG_Controller.ctrlKeyForStart = event.ctrlKey;
        else
            gDBG_Controller.ctrlKeyForStart = null;
        return command_doCommand(command)
    } catch(e) {
        DBG.log.exception(e);
        window.updateCommands('debug_state');
    }
    return false;
}

function DebuggerItemsController() {
    this.ctrlKeyForStart = 0;
    window.controllers.appendController(this);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
DebuggerItemsController.prototype = new Controller();
DebuggerItemsController.prototype.constructor = DebuggerItemsController;

DebuggerItemsController.prototype.destroy = function() {
    window.controllers.removeController(this);
}

DebuggerItemsController.prototype._debuggerCanStart = function() {
    var v = gViewMgr.currentView;
    return (!DBG.currentSession || DBG.currentSession.isInteractive) &&
            v && v.getAttribute('type') == 'editor' &&
            v.document && v.document.file && v.document.file.isLocal;
}

DebuggerItemsController.prototype._debuggerSessionCanStep = function() {
    return DBG.currentSession &&
           !DBG.currentSession.isInteractive &&
           DBG.currentSession.canStep();
}

DebuggerItemsController.prototype._debuggerCanStepOrStart = function() {
    return this._debuggerCanStart() ||
           this._debuggerSessionCanStep();
}

DebuggerItemsController.prototype._debuggerCanRun = function() {
    return this._debuggerCanStart() ||
            (DBG.currentSession && !DBG.currentSession.isInteractive &&
             DBG.currentSession.canRun());
}

DebuggerItemsController.prototype._debuggerSessionStateIs = function (state) {
    return DBG.currentSession &&
           !DBG.currentSession.isInteractive &&
           DBG.currentSession.getState() == state;
}

DebuggerItemsController.prototype._debuggerSessionStateIsNot = function (state) {
    return DBG.currentSession &&
           !DBG.currentSession.isInteractive &&
           DBG.currentSession.getState() != state;
}

DebuggerItemsController.prototype.is_cmd_dbgNewSession_enabled = function () {
    return DBG.currentSession;
}

DebuggerItemsController.prototype.do_cmd_dbgNewSession = function() {
    /* setup a new debugger tab, then step into the current buffer */
    gOutputTabManager.showNewTab();
    return DBG.doStep(Components.interfaces.koIDBGPSession.RESUME_STEP_IN);
}

DebuggerItemsController.prototype.is_cmd_dbgStepIn_enabled = function () {
    return this._debuggerCanStepOrStart();
}

DebuggerItemsController.prototype.do_cmd_dbgStepIn = function() {
    // if the current session is interactive, start a new debugger tab
    if (DBG.currentSession && DBG.currentSession.isInteractive) {
        gOutputTabManager.showNewTab();
    }
    return DBG.doStep(Components.interfaces.koIDBGPSession.RESUME_STEP_IN);
}

DebuggerItemsController.prototype.is_cmd_dbgStepOver_enabled = function () {
    return this._debuggerSessionCanStep();
}

DebuggerItemsController.prototype.do_cmd_dbgStepOver = function() {
    return DBG.doStep(Components.interfaces.koIDBGPSession.RESUME_STEP_OVER);
}

DebuggerItemsController.prototype.is_cmd_dbgStepCursor_enabled = function () {
    return this._debuggerCanStepOrStart();
}

DebuggerItemsController.prototype.do_cmd_dbgStepCursor = function() {
    return DBG.doStep(Components.interfaces.koIDBGPSession.RESUME_GO, true);
}

DebuggerItemsController.prototype.is_cmd_dbgStepOut_enabled = function () {
    return this._debuggerSessionCanStep();
}
DebuggerItemsController.prototype.do_cmd_dbgStepOut = function() {
    return DBG.doStep(Components.interfaces.koIDBGPSession.RESUME_STEP_OUT);
}

DebuggerItemsController.prototype.is_cmd_dbgDetach_enabled = function () {
    return this._debuggerSessionStateIsNot(DBG.STATE_STOPPED) &&
           DBG.currentSession.dbg &&
           DBG.currentSession.dbg.supportsDetach;
}
DebuggerItemsController.prototype.do_cmd_dbgDetach = function() {
    return DBG.currentSession.detach();
}

DebuggerItemsController.prototype.is_cmd_dbgBreakNow_enabled = function () {
    return this._debuggerSessionStateIs(DBG.STATE_RUNNING);
}
DebuggerItemsController.prototype.do_cmd_dbgBreakNow = function() {
    return DBG.currentSession.breaknow();
}

DebuggerItemsController.prototype.is_cmd_dbgStop_enabled = function () {
    return this._debuggerSessionStateIsNot(DBG.STATE_STOPPED);
}
DebuggerItemsController.prototype.do_cmd_dbgStop = function() {
    // user forcing a stop, dont reset the view on shutdown.  this is done here
    // since the terminate function is used in other cases as well, and we only
    // want to do this if the user asked for it
    DBG.currentSession.viewAtDebuggerStart = null;
    return DBG.currentSession.terminate();
}

DebuggerItemsController.prototype.is_cmd_dbgGo_enabled = function () {
    return this._debuggerCanStepOrStart();
}
DebuggerItemsController.prototype.do_cmd_dbgGo = function() {
    // if the current session is interactive, start a new debugger tab
    if (DBG.currentSession && DBG.currentSession.isInteractive) {
        gOutputTabManager.showNewTab();
    }
    return DBG.doStep(Components.interfaces.koIDBGPSession.RESUME_GO);
}

DebuggerItemsController.prototype.is_cmd_dbgRun_enabled = function () {
    return this._debuggerCanRun();
}
DebuggerItemsController.prototype.do_cmd_dbgRun = function() {
    // if the current session is interactive, start a new debugger tab
    if (DBG.currentSession && DBG.currentSession.isInteractive) {
        gOutputTabManager.showNewTab();
    }
    return DBG_Invocation.runScript();
}

DebuggerItemsController.prototype.is_cmd_dbgShowCurrentStatement_enabled = function () {
    return this._debuggerSessionStateIs(DBG.STATE_BREAK) ||
           this._debuggerSessionStateIs(DBG.STATE_INTERACTIVE);
}
DebuggerItemsController.prototype.do_cmd_dbgShowCurrentStatement = function() {
    DBG.currentSession.showCurrentStatement();
}
DebuggerItemsController.prototype .is_cmd_dbgBreakpointToggle_enabled = function () {
    return (gViewMgr.currentView &&
            gViewMgr.currentView.getAttribute('type') == 'editor' &&
            gViewMgr.currentView.document &&
            gViewMgr.currentView.document.file);
            // XXX we need to allow breakpoints to be set in XML files
            //&& gViewMgr.currentView.document.languageObj.debuggable);
}
DebuggerItemsController.prototype.do_cmd_dbgBreakpointToggle = function() {
    var view = gViewMgr.currentView;
    var line = view.scimoz.lineFromPosition(view.scimoz.currentPos) + 1;
    Breakpoints_ToggleBreakpointState(view, line);
}
DebuggerItemsController.prototype .is_cmd_dbgBreakpointAddOrEdit_enabled = function () {
    return (gViewMgr.currentView &&
            gViewMgr.currentView.getAttribute('type') == 'editor' &&
            gViewMgr.currentView.document &&
            gViewMgr.currentView.document.file);
            // XXX we need to allow breakpoints to be set in XML files
            //&& gViewMgr.currentView.document.languageObj.debuggable);
}
DebuggerItemsController.prototype.do_cmd_dbgBreakpointAddOrEdit = function() {
    try {
        var view = gViewMgr.currentView;
        var uri = view.document.file.URI;
        var line = view.scimoz.lineFromPosition(view.scimoz.currentPos) + 1;
        var bp = gBreakpointManager.getBreakpointAtFileAndLine(uri, line);
        if (bp == null) {
            Breakpoints_NewBreakpoint();
        } else {
            Breakpoints_BreakpointProperties(bp);
        }
    } catch (ex) {
        log.exception(ex);
    }
}
DebuggerItemsController.prototype .is_cmd_dbgSpawnpointAddOrEdit_enabled = function () {
    return (gViewMgr.currentView &&
            gViewMgr.currentView.getAttribute('type') == 'editor' &&
            gViewMgr.currentView.document &&
            gViewMgr.currentView.document.file &&
            gViewMgr.currentView.document.languageObj.name == "Tcl");
}
DebuggerItemsController.prototype.do_cmd_dbgSpawnpointAddOrEdit = function() {
    try {
        var view = gViewMgr.currentView;
        var uri = view.document.file.URI;
        var line = view.scimoz.lineFromPosition(view.scimoz.currentPos) + 1;
        var sp = gBreakpointManager.getSpawnpointAtFileAndLine(uri, line);
        if (sp == null) {
            Breakpoints_NewSpawnpoint();
        } else {
            Breakpoints_BreakpointProperties(sp);
        }
    } catch (ex) {
        log.exception(ex);
    }
}
DebuggerItemsController.prototype .is_cmd_dbgBreakpointClearAllInURI_enabled = function () {
    return (gViewMgr.currentView &&
            gViewMgr.currentView.getAttribute('type') == 'editor' &&  // skip browser views
            gViewMgr.currentView.document &&
            gViewMgr.currentView.document.file);
            // XXX we need to allow breakpoints to be set in XML files
            //&& gViewMgr.currentView.document.languageObj.debuggable);
}
DebuggerItemsController.prototype.do_cmd_dbgBreakpointClearAllInURI = function() {
    var uri = gViewMgr.currentView.document.file.URI;
    Breakpoints_DeleteAllBreakpointsInURI(uri);
}

// supported handles the greyed out of the menu item
DebuggerItemsController.prototype.is_cmd_debuggerListener_supported = function() {
    return true;
}
// enabled is the checkmark
DebuggerItemsController.prototype.is_cmd_debuggerListener_enabled = function() {
    // pref listenForDebugger
    return gDBG_Listener.isListening();
}
// called on changing the state of the menu checkmark
DebuggerItemsController.prototype.do_cmd_debuggerListener = function() {
    // listenfordebugger pref is for starting listening when komodo starts
    if (gDBG_Listener.isListening()) {
        gDBG_Listener.stopListener();
        gDBG_Listener.stopOnConnect = true;
    } else {
        gDBG_Listener.startListener();
        gDBG_Listener.stopOnConnect = false;
    }
}

// supported handles the greyed out of the menu item
DebuggerItemsController.prototype.is_cmd_dbgInteractive_supported = function() {
    return DBG.currentSession && DBG.currentSession.dbg.supportsInteract &&
            (DBG.currentSession.getState() == DBG.STATE_INTERACTIVE ||
            DBG.currentSession.getState() == DBG.STATE_BREAK);
}
DebuggerItemsController.prototype.is_cmd_dbgInteractive_enabled = function () {
    return DBG.currentSession && DBG.currentSession.dbg.supportsInteract
              && DBG.currentSession.getState() == DBG.STATE_INTERACTIVE;
}
DebuggerItemsController.prototype.do_cmd_dbgInteractive = function() {
    /* setup a new debugger tab, then step into the current buffer */
    try {
        if (DBG.currentSession.getState() != DBG.STATE_INTERACTIVE) {
            window.setTimeout('DBG.currentSession.enterInteractiveMode();',0);
        } else {
            window.setTimeout('DBG.currentSession.leaveInteractiveMode();',0);
        }
    } catch(e) {
        DBG.log.exception(e);
    }
}

DebuggerItemsController.prototype.is_cmd_dbgInteractiveClearBuffer_supported = function() {
    return DBG.currentSession && DBG.currentSession.dbg.supportsInteract &&
            (DBG.currentSession.getState() == DBG.STATE_INTERACTIVE ||
            DBG.currentSession.getState() == DBG.STATE_BREAK);
}
DebuggerItemsController.prototype.is_cmd_dbgInteractiveClearBuffer_enabled = function () {
    return DBG.currentSession && DBG.currentSession.dbg.supportsInteract
              && DBG.currentSession.getState() == DBG.STATE_INTERACTIVE;
}
DebuggerItemsController.prototype.do_cmd_dbgInteractiveClearBuffer = function() {
    /* setup a new debugger tab, then step into the current buffer */
    try {
        DBG.currentSession.panel.tabPanel.terminalView.clearBuffer();
    } catch(e) {
        DBG.log.exception(e);
    }
}

// supported handles the greyed out of the menu item
DebuggerItemsController.prototype.is_cmd_dbgShowHiddenVars_supported = function() {
    return DBG.currentSession && DBG.currentSession.dbg.supportsHiddenVars &&
            (DBG.currentSession.getState() == DBG.STATE_INTERACTIVE ||
            DBG.currentSession.getState() == DBG.STATE_BREAK);
}
DebuggerItemsController.prototype.is_cmd_dbgShowHiddenVars_enabled = function () {
    return DBG.currentSession && DBG.currentSession.showHiddenVars;
}
DebuggerItemsController.prototype.do_cmd_dbgShowHiddenVars = function() {
    /* setup a new debugger tab, then step into the current buffer */
    try {
        if (DBG.currentSession.showHiddenVars) {
            DBG.currentSession.showHiddenVars = false;
        } else {
            DBG.currentSession.showHiddenVars = true;
        }
        DBG.currentSession.panel.tabPanel.updateVariables();
        return true;
    } catch(e) {
        DBG.log.exception(e);
    }
    return false;
}

DebuggerItemsController.prototype.is_cmd_dbgViewAsHTML_enabled = function () {
    // only add to the watch panel
    try {
    return DBG.currentSession &&
        DBG.currentSession.panel.tabPanel.outputTabs.selectedItem.getAttribute('id') == 'html-tab';
    } catch (e) {
        log.exception(e);
    }
    return false;
}

DebuggerItemsController.prototype.do_cmd_dbgViewAsHTML = function() {
    try {
        DBG.currentSession.panel.loadHTMLView();
        return  true;
    } catch (e) {
        log.exception(e);
    }
    return false;
}


// XXX these may need their own controller, as they should update when var tabs are switched
DebuggerItemsController.prototype.is_cmd_dbgAddVariable_enabled = function () {
    // only add to the watch panel
    if (!gOutputTabManager) return false;
    var panel = gOutputTabManager.getCurrentTab();
    return panel && panel.tabPanel.variablesTabpanels.selectedPanel ==
           panel.tabPanel.variablesTabpanels.firstChild;
}

DebuggerItemsController.prototype.do_cmd_dbgAddVariable = function() {
    try {
    if (!gOutputTabManager) return false;
    var panel = gOutputTabManager.getCurrentTab();
    if (!panel) return false;
    return panel.tabPanel.watchVariablesTab.panel.firstChild.newVar();
    } catch (e) {
        log.exception(e);
    }
    return false;
}

DebuggerItemsController.prototype.is_cmd_dbgEditVariable_enabled = function () {
    try {
    if (!gOutputTabManager) return false;
    var panel = gOutputTabManager.getCurrentTab();
    return panel && panel.tabPanel.variablesTabpanels.selectedPanel.firstChild.tree.currentIndex >= 0;
    } catch(e) {
        //log.exception(e);
    }
    return false;
}

DebuggerItemsController.prototype.do_cmd_dbgEditVariable = function() {
    if (!gOutputTabManager) return false;
    var panel = gOutputTabManager.getCurrentTab();
    if (!panel) return false;
    return panel.tabPanel.variablesTabpanels.selectedPanel.firstChild.editVar(null, 'value');
}

DebuggerItemsController.prototype.is_cmd_dbgWatchedVariable_enabled = function () {
    try {
    if (!gOutputTabManager) return false;
    var panel = gOutputTabManager.getCurrentTab();
    return panel && panel.tabPanel.variablesTabpanels.selectedPanel.firstChild.tree.currentIndex >= 0;
    } catch(e) {
        //log.exception(e);
    }
    return false;
}

DebuggerItemsController.prototype.do_cmd_dbgWatchedVariable = function() {
    if (!gOutputTabManager) return false;
    var panel = gOutputTabManager.getCurrentTab();
    if (!panel) return false;
    return panel.tabPanel.variablesTabpanels.selectedPanel.firstChild.editVar(null, 'name');
}

DebuggerItemsController.prototype.is_cmd_dbgMakeWatchedVariable_enabled = function () {
    try {
    if (!gOutputTabManager) return false;
    var panel = gOutputTabManager.getCurrentTab();
    return panel && panel.tabPanel.variablesTabpanels.selectedPanel.firstChild.tree.currentIndex >= 0;
    } catch(e) {
        //log.exception(e);
    }
    return false;
}

DebuggerItemsController.prototype.do_cmd_dbgMakeWatchedVariable = function() {
    /* setup a new debugger tab, then step into the current buffer */
    try {
    if (!gOutputTabManager) return false;
        var panel = gOutputTabManager.getCurrentTab();
        if (!panel) return false;
        var names = panel.tabPanel.variablesTabpanels.selectedPanel.firstChild.getSelectedVariables();
        // make the watched tab selected
        panel.tabPanel.variablesTabs.selectedItem = panel.tabPanel.watchVariablesTab;
        for (var i=0; i < names.length; i++) {
            panel.tabPanel.watchVariablesTab.panel.firstChild.newVar(names[i]);
        }
        return true;
    } catch(e) {
        //log.exception(e);
    }
    return false;
}

DebuggerItemsController.prototype.is_cmd_dbgDeleteVariable_enabled = function () {
    // only delete from the watch panel
    try {
    if (!gOutputTabManager) return false;
    var panel = gOutputTabManager.getCurrentTab();
    return panel && panel.tabPanel.variablesTabpanels.selectedPanel ==
           panel.tabPanel.variablesTabpanels.firstChild &&
           panel.tabPanel.variablesTabpanels.selectedPanel.firstChild.tree.currentIndex >= 0;
    } catch(e) {
        log.exception(e);
    }
    return false;
}

DebuggerItemsController.prototype.do_cmd_dbgDeleteVariable = function() {
    if (!gOutputTabManager) return false;
    var panel = gOutputTabManager.getCurrentTab();
    if (!panel) return false;
    return panel.tabPanel.watchVariablesTab.panel.firstChild.deleteVar();
}

DebuggerItemsController.prototype.is_cmd_toggleInteractiveShell_enabled = function () {
    var language = getDefaultInteractiveShellLanguage();
    var tabMgr = gOutputTabManager.getInteractiveShell(language);
    if (!tabMgr) {
        switch (language) {
        case 'Python':
            return this.is_cmd_startPythonInteractiveShell_enabled();
        case 'Perl':
            return this.is_cmd_startPerlInteractiveShell_enabled();
        case 'Ruby':
            return this.is_cmd_startRubyInteractiveShell_enabled();
        case 'Tcl':
            return this.is_cmd_startTclInteractiveShell_enabled();
        }
        return false;
    }
    return true;
}

DebuggerItemsController.prototype.do_cmd_toggleInteractiveShell = function() {
    return DBG_Invocation.toggleInteractiveShell();
}


DebuggerItemsController.prototype.is_cmd_startInteractiveShell_enabled = function () {
    /* XXX we should do this, but it disables the button menu
    var language = gPrefs.getStringPref('interactiveShellDefaultLanguage');
    switch (language) {
    case 'Python':
        return this.is_cmd_startPythonInteractiveShell_enabled();
    case 'Perl':
        return this.is_cmd_startPerlInteractiveShell_enabled();
    case 'Ruby':
        return this.is_cmd_startRubyInteractiveShell_enabled();
    case 'Tcl':
        return this.is_cmd_startTclInteractiveShell_enabled();
    }
    return false;
    */
    return true;
}
DebuggerItemsController.prototype.do_cmd_startInteractiveShell = function() {
    return DBG_Invocation.interactiveShell();
}

// language specific commands for built in menu items
DebuggerItemsController.prototype.is_cmd_startPythonInteractiveShell_enabled = function () {
    return Components.classes["@activestate.com/koPythonInfoEx;1"].
                getService(Components.interfaces.koIAppInfoEx).executablePath != null;
}
DebuggerItemsController.prototype.do_cmd_startPythonInteractiveShell = function() {
    return DBG_Invocation.interactiveShell('Python');
}

// language specific commands for built in menu items
DebuggerItemsController.prototype.is_cmd_startPerlInteractiveShell_enabled = function () {
    return Components.classes["@activestate.com/koPerlInfoEx;1"].
                getService(Components.interfaces.koIAppInfoEx).executablePath != null;
}
DebuggerItemsController.prototype.do_cmd_startPerlInteractiveShell = function() {
    return DBG_Invocation.interactiveShell('Perl');
}

// language specific commands for built in menu items
DebuggerItemsController.prototype.is_cmd_startRubyInteractiveShell_enabled = function () {
    return Components.classes["@activestate.com/koRubyInfoEx;1"].
                getService(Components.interfaces.koIAppInfoEx).executablePath != null;
}
DebuggerItemsController.prototype.do_cmd_startRubyInteractiveShell = function() {
    return DBG_Invocation.interactiveShell('Ruby');
}

// language specific commands for built in menu items
DebuggerItemsController.prototype.is_cmd_startTclInteractiveShell_enabled = function () {
    return Components.classes["@activestate.com/koTclInfoEx;1"].
                getService(Components.interfaces.koIAppInfoEx).tclsh_path != null;
}
DebuggerItemsController.prototype.do_cmd_startTclInteractiveShell = function() {
    return DBG_Invocation.interactiveShell('Tcl');
}
