/* Copyright (c) 2003-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* A dialog to notify the user of an unexpected Komodo error and to encourage
 * them to report it.
 *
 * Usage:
 *  All dialog interaction is done via an object passed in and out as the first
 *  window argument: window.arguments[0].
 *      .error          A short description of the error.
 *      .text           A string of text that will be displayed in a
 *                      non-edittable selectable text box. Generally this is
 *                      some text that the user is encouraged to quote in their
 *                      bug report.
 *      .bnamespace     (optional) a namespace that supports
 *                      browse_OpenUrlInDefaultBrowser(). Defaults to opener.
 */

var log = getLoggingMgr().getLogger("dialogs.internalError");
//log.setLevel(LOG_DEBUG);

var gBNamespace = null;  // a JavaScript namespace that has browse_*() support


//---- interface routines for XUL

function OnLoad()
{
    try {
        var dialog = document.getElementById("dialog-internalerror");
        var okButton = dialog.getButton("accept");
        okButton.setAttribute("accesskey", "o");

        // .error
        var errorWidget = document.getElementById("error");
        var error = window.arguments[0].error;
        var textNode = document.createTextNode(error);
        errorWidget.appendChild(textNode);

        // .text
        var textWidget = document.getElementById("text");
        var text = window.arguments[0].text;
        var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
                      getService(Components.interfaces.koIInfoService);
        var verInfo = "Komodo " + infoSvc.productType + ", version " +
                      infoSvc.version + ", build " + infoSvc.buildNumber +
                      ".\nBuilt on " + infoSvc.buildASCTime + ".";
        textWidget.removeAttribute("collapsed");
        textWidget.value = text + "\n\n" + verInfo;

        // .bnamespace
        if (typeof(window.arguments[0].bnamespace) == "undefined" || window.arguments[0].bnamespace == null) {
            gBNamespace = opener;
        } else {
            gBNamespace = window.arguments[0].bnamespace;
        }

        window.sizeToContent();
        if (opener.innerHeight == 0) { // indicator that opener hasn't loaded yet
            dialog.centerWindowOnScreen();
        } else {
            dialog.moveToAlertPosition(); // requires a loaded opener
        }
        // Otherwise the textbox is given focus and it eats <Enter>.
        okButton.focus();
    } catch(ex) {
        log.exception(ex, "Error loading 'Internal Error' dialog.");
    }
}

function Browse(url)
{
    try {
        gBNamespace.browse_OpenUrlInDefaultBrowser(url);
    } catch(ex) {
        log.exception(ex, "Error trying to open '"+url+"' in default browser.");
    }
}

