/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

// Constants and global variables
var cellparent, cellparent_value;
const VK_ENTER = 13;  // the keycode for the "Enter" key
var data = {
  associations: null,
  original: null
};
var dialog  = {}; //XXX used both for easy access to named XUL elements and to keep
// a handle on functions that we need in PrefAssociation_OkCallback(). Every
// reference to one of these functions that might be called, directly or
// indirectly, from PrefAssociation_OkCallback() must be accessed through
// the dialog object, or this will break.  For consistency, we always call
// these functions through the dialog object.

function Association(pattern, language, invocationName) {
  this.pattern = pattern;
  this.language = language;
  this.invocationName = invocationName;
  this.id = Association._generateId();
}

// Each Association object has a unique identifier.  This is used to
// establish a relationship between the association object and the
// items in the tree that represent it.

Association._index = 0;  // used for generating ids
Association._generateId = function() {
  return "assoc" + Association._index++;
}

// Handlers

function PrefAssociation_OnLoad() {
    dialog.addPatternTextfield = document.getElementById('addPatternTextfield');
    dialog.addLanguageList = document.getElementById('addLanguageList');
    dialog.addButton = document.getElementById('addButton');
    dialog.associationList = document.getElementById('associationList');
    dialog.languageList = document.getElementById('languageList');

    parent.hPrefWindow.onpageload();
}

function OnPreferencePageInitalize(prefset) {
    data.associations = getAssociationListFromPreference(prefset);
    data.original = getAssociationListFromPreference(prefset);
}

function OnPreferencePageLoading(prefset) {
  dialog.languageList.selection = document.getAnonymousNodes(dialog.languageList)[0].firstChild.firstChild.getAttribute('label');

  dialog.addLanguageList.selection = document.getAnonymousNodes(dialog.addLanguageList)[0].firstChild.firstChild.getAttribute('label');

  // fill out the associations tree - should cache this entire tree.
  // adding items while collapsed is:
  // a) much faster, and
  // b) does not crash on Linux (who knows why, but this solves it :)
  for (var i = 0; i < data.associations.length; i++) {
      associationListAddRow(data.associations[i].id,data.associations[i].pattern,data.associations[i].language);
  }
  dialog.associationList.selectedIndex = 0;
}

function OnPreferencePageOK(prefset) {
    var oldAssociations = data.original;
    var newAssociations = data.associations;

    // only write out the prefs if they have actually changed
    if (!areAssociationListsEquivalent(oldAssociations, newAssociations)) {
      var collectionPref =
           Components.classes["@activestate.com/koOrderedPreference;1"]
              .createInstance(Components.interfaces.koIOrderedPreference);
      for (var i = 0; i < newAssociations.length; i++) {
          var singlePref =
              Components.classes["@activestate.com/koOrderedPreference;1"]
                 .createInstance(Components.interfaces.koIOrderedPreference);
          singlePref.appendStringPref(newAssociations[i].pattern);
          singlePref.appendStringPref(newAssociations[i].language);
          singlePref.appendStringPref(newAssociations[i].invocationName);
          collectionPref.appendPref(singlePref);
      }
      prefset.setPref("fileAssociations", collectionPref);
    }
    return true;
}




function sort_association(a, b) {

    if (a.pattern < b.pattern) return -1;
    if (a.pattern > b.pattern) return 1;
    return 0;
}

// Return an array of Association objects, built from the fileAssociations
// preference.
function getAssociationListFromPreference(prefset) {
  var list = new Array();
  if (prefset.hasPref("fileAssociations")) {
    var fileAssociationsPref = prefset.getPref("fileAssociations");
    for (var i = 0; i < fileAssociationsPref.length; i++) {
      var tuple = fileAssociationsPref.getPref(i).QueryInterface(Components.interfaces.koIOrderedPreference);
      // invocations were added later
      var invocationName = tuple.length > 2 ? tuple.getStringPref(2) : "";
      list.push(new Association(tuple.getStringPref(0),
              tuple.getStringPref(1), invocationName));
    }
  }
  list.sort(sort_association);
  return list;
}

// Return true if the arrays of Association objects, a and b, correspond to
// the same underlying representation as a preference.
function areAssociationListsEquivalent(a, b) {
  if (a.length != b.length) return false;

  for (var i = 0; i < a.length; i++) {
    // we don't care about the IDs
    if (a[i].pattern != b[i].pattern || a[i].language != b[i].language || a[i].invocationName != b[i].invocationName) {
      return false;
    }
  }

  return true;
}

function associationListAddRow(id, pattern, language) {
    var item = dialog.associationList.appendItem(pattern, language);
    item.id = id;
    return item;
}

function onAddAssociation() {
  var pattern = dialog.addPatternTextfield.value;
  var language = dialog.addLanguageList.selection;
  var invocation = "";
  var association = new Association(pattern, language, invocation);

  data.associations.push(association);

  var newitem = associationListAddRow(association.id,pattern,language);

  dialog.addPatternTextfield.setAttribute('value', '');

  dialog.associationList.ensureElementIsVisible ( newitem );
  dialog.associationList.selectItem ( newitem );
  dialog.addPatternTextfield.focus();
  updateButtonStates();
  dialog.addPatternTextfield.value = '';
}

function onRemoveAssociation() {
  var kids = dialog.associationList;
  var selItem = dialog.associationList.selectedItem;  // treeitem
  var pattern = selItem.getAttribute('label');
  if (!pattern) return;
  var index = getPatternIndex(pattern);
  // remove the item from the association array
  data.associations.splice(index, 1);

  // Remove the item from the tree
  var nextUp;
  if (selItem.nextSibling)
      nextUp = selItem.nextSibling;
  else if (selItem.previousSibling)
      nextUp = selItem.previousSibling;
  kids.removeChild(selItem);
  dialog.associationList.selectedItem = nextUp;
  cellparent = nextUp;
}

function selectAssociation(event)  {
    if (!dialog.associationList || dialog.associationList.selectedIndex < 0) return true;
    var assoc = data.associations[dialog.associationList.selectedIndex];
    dialog.languageList.selection = assoc.language;
    return true;
}

function updateAssociation()  {
      if (dialog.associationList.selectedIndex >= 0)  {
          data.associations[dialog.associationList.selectedIndex].language =
                        dialog.languageList.selection;
      }
}

function updateButtonStates() {
  var pattern = trimString(dialog.addPatternTextfield.value);
  if (!pattern || getPatternIndex(pattern) != null) {
    dialog.addButton.setAttribute('disabled', 'true');
  } else {
    dialog.addButton.removeAttribute('disabled');
  }
}

// DOM construction helpers

function getPatternIndex(pattern) {
  for (var i = 0; i < data.associations.length; i++) {
    if (pattern == data.associations[i].pattern) return i;
  }
  return null;
}

function getIdIndex(id) {
  for (var i = 0; i < data.associations.length; i++) {
    if (id == data.associations[i].id) return i;
  }
  return null;
}

// Remove whitespace from both ends of a string
function trimString(string)
{
  if (!string) return "";
  return string.replace(/(^\s+)|(\s+$)/g, '')
}
