/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/*

base Manager

base class for all 'project' viewer manager classes

*/
var gPartManagers = [];

function baseManager(command_name) {
    this.viewMgr = null; // set in subclasses
    this._currentitem = null;
    this._command_name = null;
    if (typeof(command_name) != 'undefined')
        this._command_name = command_name;
    this.log = getLoggingMgr().getLogger('baseManager');
}

baseManager.prototype.constructor = baseManager;

baseManager.prototype.getCurrentItem = function() {
    return this._currentitem;
}

baseManager.prototype.setCurrentItem = function(item) {
    this._currentitem = item;
}

baseManager.prototype.addItem = function(/* koIPart */ part, index) {
    this.log.error('baseManager.prototype.addItem not implemented');
}

baseManager.prototype.removeItem = function(item, skipdialog) {
    if (typeof(skipdialog)=='undefined') skipdialog = false;
    var container;
    if (this == toolboxMgr) {
        container = 'Toolbox';
    } else {
        container = " project '" + item.part.project.name + "'";
    }
    var question = "Do you want to remove the " + item.type +
                   " '" +
                   item.name + "' from the " + container + "?";
    if (skipdialog || dialog_yesNo(question) == "Yes") {
        /* remember to possibly unset current item */
        if (this.getCurrentItem() == item) {
            this.setCurrentItem(null);
        }

        try {
            this.viewMgr.view.removePart(item);
        } catch (e) {
            this.log.error(e);
        }
        return true;
    }
    return false;
}

baseManager.prototype.renameItem = function(item) {
    var resp = new Object ();
    resp.name= item.name;
    resp.value = item.value;
    resp.res = '';

    if (resp.value) {
        openWindowUniqueInstance("chrome://jaguar/content/project/renamePart.xul",
                'komodo_renamePart',
                "chrome,dialog,modal,resizable=yes",
                resp);
        if (resp.res != true) {
            return false;
        }
    } else {
        resp.name = dialog_prompt(
            "Enter the new name.", // prompt
            null, // label
            item.name, // default
            "New Name"); // title
        if (!resp.name) return false;
    }
    /* Modify the project attribute and the wrapped attribute */
    item.name = resp.name;
    return true;
}

baseManager.prototype.getItemsByURL = function(url) {
    this.log.error('baseManager.prototype.getItemsByURL not implemented');
}

baseManager.prototype.refreshView = function() {
    if (this.viewMgr.view.mTree) {
        this.viewMgr.view.mTree.beginUpdateBatch();
        this.viewMgr.view.mTree.invalidate();
        this.viewMgr.view.mTree.endUpdateBatch();
    }
}

baseManager.prototype.invalidateItem = function(item) {
    var index = this.viewMgr.view.getIndexByPart(item);
    if (index >= 0) {
        this.viewMgr.view.invalidateRow(index);
        return true;
    }
    return false;
}

baseManager.prototype.findChildByAttributeValue = function (item, attribute, value) {
    if (!item.hasWrappedChildren()) return null;
    var found = null;
    for (var i in item.children) {
        var child = item.children[i];
        if (child.part.hasAttribute(attribute) && child.part.getStringAttribute(attribute) == value) {
            return child;
        }
        found = this.findChildByAttributeValue(child,attribute,value);
        if (found) return found;
    }
    return null;
}


baseManager.prototype.isChildOf = function (parent, child) {
    if (!parent.hasWrappedChildren()) return null;
    for (var i in parent.children) {
        if (parent.children[i] == child) return true;
        if (parent.children[i].hasWrappedChildren()) {
            if (this.isChildOf(parent.children[j], child)) return true;
        }
    }
    return false;
}

baseManager.prototype.findItemByURL = function (parent, type, url) {
    var i;
    var j;
    var item;
    var child;
    if (!parent.hasWrappedChildren()) return null;
    if (parent.children) {
        for (i in parent.children) {
            item = parent.children[i];
            if (!type || item.type == type) {
                if (item.url == url || (item.part.hasAttribute('url')
                    && item.part.getStringAttribute('url') == url)) {
                    return item;
                }
            }
            if (item.type == 'folder') {
                for (j in parent.children) {
                    child = this.findItemByURL(parent.children[j], type, url);
                    if (child != null) {
                        return child;
                    }
                }
            }
        }
    }
    if (parent.url == url || (parent.part.hasAttribute('url')
            && parent.part.getStringAttribute('url') == url)) {
        return parent;
    }
    return null;
}

baseManager.prototype.applyPartKeybindings = function () {
    // No default implementation -- up to each partManager to define it,
    // using the callback in the wrappedPart.
}

baseManager.prototype.writeable = function () {
    // No default implementation -- up to each partManager to define it,
}

baseManager.prototype.updateMenuForInterface = function(defaultMenuInterface,menuEl) {
    this.log.error('baseManager.prototype.updateMenuForInterface not implemented');
}

/* looks in all managers for items related to the url so it can update them */
function findItemsByURL(url)
{
    var items = [];
    var list;
    for (var i=0; i < gPartManagers.length; i++) {
        list = gPartManagers[i].getItemsByURL(url);
        items = items.concat(list);
    }
    return items;
}

function removeItemsByURL(url, skipDialog)
{
    var items = [];
    var i, j, list;
    for (i=0; i < gPartManagers.length; i++) {
        list = gPartManagers[i].getItemsByURL(url);
        for (j = 0; j < list.length; j++) {
            gPartManagers[i].removeItem(list[j], skipDialog);
        }
    }
    return items;
}

/* looks in all managers for _parts_ related to the url so it can update them */
function findPartsByURL(url)
{
    var parts = []
    for (var i=0; i < gPartManagers.length; i++) {
        var list = gPartManagers[i].getPartsByURL(url);
        parts = parts.concat(list);
    }
    return parts;
}

function invalidateItem(item)
{
    for (var i=0; i < gPartManagers.length; i++) {
        if (gPartManagers[i].invalidateItem(item)) continue;
    }
}
