/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

// start the extension
registeredExtensions[registeredExtensions.length] = new peFolder();

function peFolder() {
    this.name = 'peFolder';
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
peFolder.prototype.constructor = peFolder;

peFolder.prototype.init = function() {
}

peFolder.prototype.registerCommands = function() {
    extensionManager.registerCommand('cmd_delete', this);
    extensionManager.registerCommand('cmd_importFromFS', this);
    extensionManager.registerCommand('cmd_importFromFS_Project', this);
    extensionManager.registerCommand("cmd_importFromPackage",this);
}

peFolder.prototype.registerMenus = function() {
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Import from File System...','cmd_importFromFS');
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Import Package...','cmd_importFromPackage');
    extensionManager.createMenuItem(Components.interfaces.koIPart_folder,'Refresh Status','cmd_refreshStatus');
    var menupopup = document.getElementById('folder_context');
    extensionManager.addMenuItem(Components.interfaces.koIPart_folder, menupopup);
}

peFolder.prototype.registerEventHandlers = function() {
    extensionManager.addEventHandler(Components.interfaces.koIPart,'onUpdatePart',this);
}

peFolder.prototype.supportsCommand = function(command, item) {
try {
    //dump('for ' + command + '\n');
    if (!gFocusedProjectView) return false;
    var items = gFocusedProjectView.getSelectedItems();
    var havemultiple = items.length > 1;
    //dump('for ' + command + ', items.length == ' + items.length + '\n');
    switch (command) {
    case 'cmd_delete':
        //HACK: Current behaviour for read-only toolbox's (typically the
        //      Shared Toolbox is to allow modifications and deal with the
        //      "can't save it" issues on save. Because adds and edits are
        //      allowed, it doesn't make sense to disallow delete. So we
        //      won't for now.
        //return gFocusedProjectView.manager.writeable();
        return true;
    case "cmd_importFromFS_Project":
    case "cmd_importFromPackage":
    case "cmd_importFromFS":
        return gProjectManager.getCurrentProject() != null;
    default:
        break;
    }
} catch (e) {
    log.exception(e);
}
    return false;
}

peFolder.prototype.isCommandEnabled = function(command, item) {
try {
    //dump('for ' + command + '\n');
    var items = null;

    if (gFocusedProjectView) {
        items = gFocusedProjectView.getSelectedItems();
        var havemultiple = items.length > 1;
    }
    switch (command) {
        case 'cmd_delete':
            // Delete is enabled for any part selection that isn't a project.
            if (view_getFocusedProjectView()) {
                if (!gFocusedProjectView.manager.writeable()) return false;
                item = gFocusedProjectView.getSelectedItem();
                if (!item) return false;
                try {
                    var i = item.part.QueryInterface(Components.interfaces.koIProject);
                    return false;
                } catch (e) {
                    // if the interface is not supported, it is not a project, so return true.
                    //dump("not a project, saying yes we can delete\n");
                    return true;
                }
            }
            // leave cmd_delete enabled for any other element
            return true;
        case "cmd_importFromFS_Project":
        case "cmd_importFromPackage":
        case "cmd_importFromFS":
            return gProjectManager.getCurrentProject() != null;
        default:
            break;
    }
} catch (e) {
    log.exception(e);
}
    return false;
}

peFolder.prototype.doCommand = function(command) {
    var item = null;
    var firstchild;
    switch (command) {
    case 'cmd_addFilePart_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addFilePart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        this.addFile(item);
        break;
    case 'cmd_addNewFile':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        this.addNewFile(item);
        break;
    case 'cmd_addRemoteFilePart_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addRemoteFilePart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        peFolder_addRemoteFile(item);
        break;
    case 'cmd_addFolderPart_Project':
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
        // fall through
    case 'cmd_addFolderPart':
        if (!item) item = gFocusedProjectView.getSelectedItem();
        var fitem = new Object();
        fitem.name = 'New Folder';
        fitem.value = null;
        if (gFocusedProjectView.manager.renameItem(fitem)) {
            this.addFolder(fitem.name, item);
        }
        break;
    case 'cmd_delete':
        // this cmd_delete handler only implements for our part viewers
        if (!view_getFocusedProjectView()) return;
        var items = gFocusedProjectView.getSelectedItems();
        var havemultiple = (items.length > 1);
        if (havemultiple) {
            var question = "Do you want to remove the " + items.length +
                           " items you have selected?";
            if (dialog_yesNo(question) != "Yes")
                return;
        }
        for (var i = 0; i < items.length; i++) {
            gFocusedProjectView.manager.removeItem(items[i], havemultiple);
        }
        break;
    case "cmd_importFromFS_Project":
        gProjectManager.viewMgr.focus();
        item = gProjectManager.getCurrentProject();
    case "cmd_importFromFS":
        if (!item) item = gFocusedProjectView.getSelectedItem();
        if (!item) return;
        part_ImportFromFS(item.part);
        firstchild = item.updateWrappedPart();
        gFocusedProjectView.refresh(item);
        gFocusedProjectView.selectPart(item);
        break;
    case "cmd_importFromPackage":
        if (!item) item = gFocusedProjectView.getSelectedItem();
        if (!item) return;
        package_ImportPackage(gFocusedProjectView, item);
        firstchild = item.updateWrappedPart();
        gFocusedProjectView.refresh(item);
        gFocusedProjectView.selectPart(item);
        break;
    default:
        break;
    }
}

peFolder.prototype.addNewFile = function(item)
{
    // Get base directory from item.  It can be a folder or a project.
    var defaultDirectory = null;
    var url;
    if (item) {
        url = item.part.getStringAttribute('url')
        var pathSvc = Components.classes["@activestate.com/koOsPath;1"]
                                .getService(Components.interfaces.koIOsPath);
        defaultDirectory = uriparse_URIToLocalPath(url);
        if (item.part.type == 'project') {
            defaultDirectory = pathSvc.dirname(defaultDirectory);
        }
    }
    var newfilename = filepicker_saveFile(defaultDirectory);
    if (!newfilename) return;
    // We need to create an empty file, add it to the 'parentitem' and open it.
    var sysUtils = Components.classes["@activestate.com/koSysUtils;1"]
                            .createInstance(Components.interfaces.koISysUtils);
    sysUtils.Touch(newfilename);
    url = uriparse_localPathToURI(newfilename);
    peFolder_addFileWithURL(url, item);
    open_openURI(newfilename);
}

function peFolder_addFileWithURL(url, item)
{
    var index = -1;
    if (typeof(item)=='undefined' || !item)
        item = gFocusedProjectView.getSelectedItem();
    if (item) index = gFocusedProjectView.getIndexByPart(item);
    var part;
// #if PRODUCT_TYPE == "professional"
    if (url.match(/\.ui$/i)) {
        part = Components.classes["@activestate.com/koPart?type=dialog;1"]
                                    .createInstance(Components.interfaces.koIPart_dialog);
    } else {
// #endif
        part = Components.classes["@activestate.com/koPart?type=file;1"]
                                    .createInstance(Components.interfaces.koIPart_file);
// #if PRODUCT_TYPE == "professional"
    }
// #endif
    part.setStringAttribute('url', url);
    part.setStringAttribute('name', uriparse_baseName(url));
    gFocusedProjectView.manager.addItem(part,index);
}

peFolder.prototype.onUpdatePart = function(item,event)
{
    // event is null for onUpdatePart
    var isProject = null;
    try {
        isProject = item.part.QueryInterface(Components.interfaces.koIProject);
    } catch (e) {    }

    if (item.isContainer && !isProject) {
        if (!item.url) {
            item.iconopenstyle['projectVirtualFolderOpen'] = 1;
            item.iconstyle['projectVirtualFolderClosed'] = 1;
        } else {
            item.iconopenstyle['projectFolderOpen'] = 1;
            item.iconstyle['projectFolderClosed'] = 1;
        }
    }
}

function peFolder_add(type, menuType)
{
    try {
        var target, item, items;
        var tabs, tab;
        tabs = document.getElementById('project_toolbox_tabs');
        // menuType can be:
        //   0: from a context menu on a selected part
        //   1: from the toplevel toolbox menuor the context menu on the toolbox tab
        //   2: from the toplevel project menu
        switch (menuType) {
            case 0:
                target = gFocusedProjectView.getSelectedItem();
                break;
            case 1:
                target = toolboxMgr.viewMgr.view;
                uilayout_ensureTabShown('toolbox_tab');
                gFocusedProjectView = target;
                break;
            case 2:
                uilayout_ensureTabShown('project_tab');
                target = gProjectManager.getCurrentProject();
                gFocusedProjectView = gProjectManager.viewMgr.view;
                break;
            case 3:
                target = gSharedToolboxMgr.viewMgr.view;
                uilayout_ensureTabShown('sharedtoolbox_tab');
                gFocusedProjectView = target;
                break;
            default:
                log.error("Unknown menuType in peFolder_add: " + menuType);
        }
        // type can be:
        //   'files': add existing files
        var name;
        switch (type) {
            case 'files':
                peFolder_addFile(target);
                break;
            case 'snippet':
                peSnippet_addSnippet(target);
                break;
            case 'command':
                peCommand_addCommand(target);
                break;
            case 'dialog':
                peGUI_addDialog(target);
                break;
            case 'menu':
                peMenu_addMenu(target);
                break;
            case 'toolbar':
                peMenu_addToolbar(target);
                break;
            case 'URL':
                peURL_addURL(target);
                break;
            case 'macro':
                peMacro_addMacro(target);
                break;
            case 'remotefiles':
                peFolder_addRemoteFile(target);
                break;
            case 'template':
                peTemplate_addTemplate(target);
                break;
            case 'directoryshortcut':
                dirname = filepicker_getFolder();
                if (!dirname) return;
                peFile_addDirectoryShortcut(target, dirname);
                break;
            case 'newfolder':
                name = dialog_prompt("Enter folder name:");
                if (!name) return;
                peFolder_addFolder(name, target);
                break;
            default:
                log.error("Unknown item type called to peFolder_add: " + type);
        }
    } catch (e) {
        log.error(e);
    }
}

function peFolder_addFile(parent_item)
{
    //XXX todo: Support other resources

    var defaultDir = null;
    if (parent_item && parent_item.url) {
        defaultDir = uriparse_dirName(parent_item.url)
    }
    var files = filepicker_openFiles(defaultDir, // default dir
                                     null, // default filename
                                     "Add Files to Project"); // title
    if (files == null) {
        return false;
    } else {
        var url;
        for (var i = 0; i < files.length; ++i) {
            url = uriparse_localPathToURI(files[i]);
            peFolder_addFileWithURL(url, parent_item);
        }
        return true;
    }
}

function peFolder_addRemoteFile(item)
{
    var list = filepicker_browseForRemoteFiles(); // in fileops.js
    if (list) {
        for (var i = 0; i < list.length; i++) {
            peFolder_addFileWithURL(list[i], item);
        }
        return true;
    }
    return false;
}

function peFolder_addFolder(name, item)
{
    var index = gFocusedProjectView.getIndexByPart(item);
    var folder = Components.classes["@activestate.com/koPart?type=folder;1"]
                                    .createInstance(Components.interfaces.koIPart_folder);
    folder.name = name;
    gFocusedProjectView.manager.addItem(folder,index);
}

