/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* Toolbox Manager
 *
 * The Toolbox manager reads items from a "per-user" project file called
 * "toolbox.kpf" in the user's userDataDir. There is also a possible
 * "Shared Toolbox" (and associated manager) in Komodo Professional builds.
 *
 * The Toolboxes are heavily modeled on the Project Manager.
 */
// Dev Notes:
// - This module would better be named toolbox.js and that prefix should
//   then be used more regularly in here.

//---- globals

var toolboxMgr = null; //XXX should be gToolboxMgr
var gSharedToolboxEnabled = null;
var gSharedToolboxMgr = null;
var _gToolboxPrefObserver = null;

var gToolboxCtrl = null;

//---- Public generic Toolbox methods

function Toolbox_onLoad()
{
    toolboxMgr = new toolboxManager();
    toolboxMgr.init();

    _gToolboxPrefObserver = new _ToolboxPrefObserver();
    _gToolboxPrefObserver.init();
    gToolboxCtrl = new toolbox_toolboxController();
    window.controllers.appendController(gToolboxCtrl);

}

function Toolbox_Export() {
    var filename = filepicker_saveFile(
            null, null, // default dir and filename
            "Export To...", // title
            "Komodo Project", // default filter
            ["Komodo Project", "All"]); // filters
            // When have .ktf files changes to this:
            //"Komodo Toolbox", // default filter
            //["Komodo Toolbox", "Komodo Project", "All"]);
    if (filename == null)
        return;

    var koDirs = Components.classes["@activestate.com/koDirs;1"].
         getService(Components.interfaces.koIDirs);
    var shutil = Components.classes["@activestate.com/koShUtil;1"].
        getService(Components.interfaces.koIShUtil);

    var userDataDir = koDirs.userDataDir;
    var os = Components.classes["@activestate.com/koOs;1"].
        getService(Components.interfaces.koIOs);
    var toolboxfname = os.path.join(userDataDir, 'toolbox.kpf');
    try {
        shutil.copyfile(toolboxfname, filename);
    } catch (e) {
        // Ignore it -- we can just go along w/ an empty toolbox (see below).
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        alert("Could not make a copy of the toolbox data file: " +
              lastErrorSvc.getLastErrorMessage());
    }
}

function Toolbox_ExportItems(items) {
    var defaultfilename = "exported_toolbox.kpf";
    var filename = filepicker_saveFile(
            null, defaultfilename, // default dir and filename
            "Export Toolbox To...", // title
            "Komodo Project", // default filter
            ["Komodo Project", "All"]); // filters
            // When have .ktf files changes to this:
            //"Komodo Toolbox", // default filter
            //["Komodo Toolbox", "Komodo Project", "All"]);
    if (filename == null)
        return;

    var project = Components.classes["@activestate.com/koProject;1"]
                  .createInstance(Components.interfaces.koIProject);
    try {
        var url = uriparse_localPathToURI(filename);
        project.create();
        project.url = url;
        for (i = 0; i < items.length; i++) {
            project.addChild(items[i].part);
        }
        project.save();
    } catch(ex) {
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
            getService(Components.interfaces.koILastErrorService);
        alert('There was an error saving project "' +
              project.name + '": ' +
              lastErrorSvc.getLastErrorMessage());
    }
}

function Toolbox_ExportPackageItems(items) {
    if (typeof(items) == 'undefined' || !items || items.length < 1) {
        dialog_alert("Please select items in the toolbox to export first.");
        return;
    }
    var info = {};
    info.items = items;
    window.openDialog(
            "chrome://jaguar/content/wizards/package-export-wizard.xul",
            "komodo_wizard",
            "centerscreen,chrome,resizable,scrollbars,dialog=no,close,modal=yes",
            info);
}

function package_ImportPackage(manager, item, packageFilename) {
    if (typeof(item) == 'undefined') item = null;
    if (typeof(packageFilename) == 'undefined') packageFilename = null;
    var info = {};
    info.item = item;
    info.packageFilename = packageFilename;
    info.gFocusedProjectView = manager;
    window.openDialog(
            "chrome://jaguar/content/wizards/package-import-wizard.xul",
            "komodo_wizard",
            "centerscreen,chrome,resizable,scrollbars,dialog=no,close,modal=yes",
            info);
}

function Toolbox_ImportPackage() {
    package_ImportPackage(toolboxMgr, null);
}

//---- A common virtual base class for both toolbox managers (user and shared)
// Dev Notes:
// - Originally there was only the one "user" or "personal" toolbox (called
//   just the "Toolbox"). When the "Shared Toolbox" was added this base
//   class was factored out. There may be remnants of "Toolbox"-specific
//   stuff in here.
//

function toolboxBaseManager() {
    KomodoRegisterCanCloseHandler(this.save, this);
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
toolboxBaseManager.prototype = new baseManager();
toolboxBaseManager.prototype.constructor = toolboxBaseManager;

// Common init code for all toolboxes. This loads the toolbox data, creating
// and empty .kpf file if necessary. If the toolbox data is corrupt it is
// moved out of the way and a new empty one is created.
//
//  "prettyName" is a human readable name, useful for error messages and
//      debugging.
//  "elementId" is the id of the <partviewer/> XUL element for this toolbox.
//  "fname" is the full local path to the toolbox KPF file.
//
// This method returns null if successful. Otherwise it returns one of the
// following error strings:
//  "could not create": The toolbox file did not exist and an attempt to
//      create an empty starter one failed. The actual error is available
//      via koILastErrorService.
//  "could not load": The toolbox file could not be loaded for whatever
//      reason (perhaps it is corrupt?).
//
toolboxBaseManager.prototype._init = function(prettyName, elementid, fname) {
    try {
        this.lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                getService(Components.interfaces.koILastErrorService);
        var osPath = Components.classes["@activestate.com/koOsPath;1"].
                     getService(Components.interfaces.koIOsPath);

        this.prettyName = prettyName;
        this.log = getLoggingMgr().getLogger('toolboxManager: ' + elementid);
        //this.log.setLevel(LOG_DEBUG);
        this.log.info('initializing toolboxBaseManager');
        this._currentitem = null;
        this.viewMgr = document.getElementById(elementid);
        this.viewMgr.onLoad(this);
        // our one and only datapoint for the toolbox
        this.viewMgr.view.datapoints['Name']='Name';

        this.toolbox = Components.classes["@activestate.com/koProject;1"]
                       .createInstance(Components.interfaces.koIProject);
        this.toolboxURL = uriparse_localPathToURI(fname);

        // Create an empty toolbox file if there currently isn't one.
        if (!osPath.exists(fname)) {
            this.log.info("The "+prettyName+" at '"+fname+"' does not "+
                          "exist. Creating a new empty one.");
            try {
                this.toolbox.create();
                this.toolbox.url = this.toolboxURL;
                this.toolbox.save();
            } catch(ex) {
                this.toolbox = null;
                return "could not create";
            }
        }

        // Make sure that there is always a gFocusedProjectView
        gFocusedProjectView = this.viewMgr.view
        gPartManagers.push(this);
        this.sorttype = 'Name';
        this.sortdir = 1;
        this.wrappedproject = null;

        // Load the toolbox file.
        try {
            this.log.info("Loading "+prettyName+" from '"+this.toolboxURL+"'");
            this.toolbox.load(this.toolboxURL);
            var _partSvc = Components.classes["@activestate.com/koPartService;1"]
                    .getService(Components.interfaces.koIPartService);
            _partSvc.toolbox = this.toolbox;

        } catch(ex) {
            log.exception(ex);
            return "could not load";
        }

        this.loadFromProject(this.toolbox);

        // Find out if we're read-only or not.
        if (this.viewMgr.hasAttribute('lock_icon_id')) {
            var lockId = this.viewMgr.getAttribute('lock_icon_id');
            var lockIcon = document.getElementById(lockId);
            if (this.writeable()) {
                lockIcon.removeAttribute('readonly');
            } else {
                lockIcon.setAttribute('readonly',"chrome://jaguar/skin/images/small_lock.png");
            }
        }

        // let the status service know it has work to do
        try {
            gObserverSvc.notifyObservers(this, 'file_update_now', null);
        } catch(e) { /* exception if no listeners */ }

        return null;
    } catch (e) {
        log.exception(e);
    }
    return "An unknown exception occurred";
}

toolboxBaseManager.prototype.close = function(sortdir) {
    try {
        if (this.toolbox && !this.wrappedproject.properties['isReadOnly']) {
            this.save();
            this.toolbox.close();
        }
        this.wrappedproject = null;
    } catch (e) {
        log.exception(e);
    }
}

toolboxBaseManager.prototype.writeable = function () {
    // Return true if the current toolbox is writeable
    // Note: We don't trust
    //           this.wrappedproject.properties["isReadOnly"]
    //       because it is using a bogus algorithm (see URIlib.py). Instead we
    //       do our own checking using os.access().
    var os = Components.classes["@activestate.com/koOs;1"].
             getService(Components.interfaces.koIOs);
    var fname = uriparse_URIToLocalPath(this.toolboxURL);
    return os.access(fname, os.W_OK);
}

toolboxBaseManager.prototype.sort = function(sortdir) {
    if (!this.wrappedproject) {
        return;
    }
    // change sortdirection if we need to
    if (typeof(sortdir) != 'undefined' && sortdir) {
        this.viewMgr.view.updateSortIndicators(this.sorttype);
        this.sortdir = this.sortdir * -1;
    }

    partwrapper_sortDirection = this.sortdir;
    partwrapper_sortField = this.sorttype;

    // clear the list and reinsert the projects
    this.viewMgr.view.clearList();
    // force fetching the projects children so that they
    // will get properly sorted when we open the toolbox
    // the first time
    if (this.wrappedproject.children) {
        this.wrappedproject.sortChildren();
        this.viewMgr.view.addToplevelChildren(this.wrappedproject.children);
    }
}

toolboxBaseManager.prototype.renameItem = function(item)
{
    var ok = baseManager.prototype.renameItem.apply(this, [item]);
    if (ok) {
        this.sort(false);
    }
    return ok;
}

toolboxBaseManager.prototype.removeItem = function(item, skipdialog)
{
    if (baseManager.prototype.removeItem.apply(this, [item,skipdialog])) {
        if (item.level == 0) {
            this.wrappedproject.removeChild(item);
        }
    }
}

toolboxBaseManager.prototype.addItem = function(/* koIPart */ part, index) {
    var isproject = false;
    try {
        /* we don't allow projects to be added to the
           toolbox, but we do allow links to projects, which
           allows us to open projects from the toolbox */
        isproject = part.QueryInterface(Components.interfaces.koIProject) != null;
    } catch(e) {}

    if (isproject) {
        var p_part = Components.classes["@activestate.com/koPart?type=ProjectRef;1"]
                                        .createInstance(Components.interfaces.koIPart_ProjectRef);
        p_part.setStringAttribute('url', part.url);
        p_part.setStringAttribute('name', os.path.withoutExtension(part.getStringAttribute('name')));
        part = p_part;
    }

    try {
        var parent = this.wrappedproject;
        if (index >= 0 && this.viewMgr.view.entriesList[index].isContainer) {
            if (!this.viewMgr.view.entriesList[index].isContainer) {
                index = this.viewMgr.view.getParentIndex(index);
            }
            parent = this.viewMgr.view.entriesList[index];
        }
        parent.part.addChild(part);
        var firstchild = parent.updateWrappedPart();
        this.sort(false);
        this.viewMgr.view.selectPart(firstchild);

        // this will prevent projects from being removed from project pane
        // if the project is dragged into the toolbox
        return !isproject;
    } catch(e) { this.log.error('toolboxBaseManager addItem exception: '+e+'\n'); }
    return false;
}

toolboxBaseManager.prototype.getItemsByURL = function(url) {
    var items = [];
    var item = this.findItemByURL(this.wrappedproject, null, url);
    if (item != null) items.push(item);
    return items;
}

toolboxBaseManager.prototype.findItemByURL = function (parent, type, url) {
    var item;
    var child;
    var i;
    var j;
    if (!parent.hasWrappedChildren()) return null;
    for (i in parent.children) {
        item = parent.children[i];
        if (!type || item.type == type) {
            if (item.url == url || (item.part.hasAttribute('url')
                && item.part.getStringAttribute('url') == url)) {
                return item;
            }
        }
        if (item.type == 'folder') {
            for (j in parent.children) {
                child = this.findItemByURL(parent.children[j], type, url);
                if (child != null) {
                    return child;
                }
            }
        }
    }
    if (parent.url == url || (parent.part.hasAttribute('url')
            && parent.part.getStringAttribute('url') == url)) {
        return parent;
    }
    return null;
}

toolboxBaseManager.prototype.findItemByAttributeValue= function (attribute, value) {
    return this.findChildByAttributeValue(this.wrappedproject, attribute, value);
}

toolboxBaseManager.prototype.findPartByTypeAttributeValue= function (type, attribute, value) {
    return this.toolbox.getChildWithTypeAndStringAttribute(type, attribute, value,true);
}

toolboxBaseManager.prototype.findPartByAttributeValue= function (attribute, value) {
    return this.toolbox.getChildByAttributeValue(attribute, value, true);
}

toolboxBaseManager.prototype.applyPartKeybindings = function () {
    this.wrappedproject.applyPartKeybindings();
}

toolboxBaseManager.prototype.getPartsByURL = function (url) {
    var part = this.toolbox.getChildByAttributeValue('url', url, true);
    if (part != null) return [part];
    return [];
}

function _toolboxUpdateMainMenu(menuEl)
{
    gFocusedProjectView = toolboxMgr.viewMgr.view;
    gFocusedProjectView.manager.viewMgr.focus();
    extensionManager.updateMenuForInterface(Components.interfaces.koIContainer, menuEl);
    extensionManager.updateMenuForInterface(Components.interfaces.koIToolbox, menuEl, true);
    return true;
}

toolboxBaseManager.prototype.updateMenuForInterface = function(defaultMenuInterface,menuEl) {
    // we ignore the defaultMenuInterface, and just use the global
    // function that handles this.
    //extensionManager.updateMenuForInterface(defaultMenuInterface, menuEl);
    _toolboxUpdateMainMenu(menuEl);
}

toolboxBaseManager.prototype.loadFromProject = function(project) {
    gTimeline.enter('loadFromProject');
    // Trick to make our items have a level 0, as required by the partviiewer for close behavior (so far).
    this.wrappedproject = new partWrapper(project, -1);
    // additem checks if these are true before handling
    // creation of wrappers for children
    this.wrappedproject.isContainer = true;
    this.wrappedproject.isContainerOpen = true;
    this.sort(false);
    if (this.viewMgr.view.rowCount && this.viewMgr.view.mSelection) {
        this.viewMgr.view.mSelection.select(0);
    }
    gTimeline.leave('loadFromProject');
}

toolboxBaseManager.prototype.loadToolboxFromURL = function(url)
{
    this.toolbox = Components.classes["@activestate.com/koProject;1"]
                                .createInstance(Components.interfaces.koIProject);
    this.toolbox.load(url);
    this.loadFromProject(this.toolbox);
}

toolboxBaseManager.prototype.verifyUnchanged = function() {
    if (this.wrappedproject.file.hasChanged) {
        prompt = 'Your '+this.prettyName+' has changed outside Komodo, '+
                  'Saving it now will overwrite changes made outside Komodo. '+
                  'You can overwrite those changes, or load the changes which '+
                  'will overwrite the changes you have made in your toolbox.';
        var response = dialog_customButtons(prompt, ["Overwrite","Reload","Cancel"],"Cancel", null, this.prettyName+" has changed on disk");
        if (response == "Cancel") {
            return false;
        } else if (response == "Reload") {
            this.toolbox.revert();
            this.loadFromProject(this.toolbox);
            return false;
        }
    }
    return true;
}

// This is occassionally used as a Komodo "can close" handler, so it
// returns true if successful, false otherwise.
toolboxBaseManager.prototype.save = function() {
    if (this.toolbox && this.toolbox.isDirty) {
        try {
            if (!this.verifyUnchanged()) return false;
            this.toolbox.save();
        } catch(ex) {
            var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                getService(Components.interfaces.koILastErrorService);
            var answer = dialog_yesNoCancel(
                "There was an error saving the "+this.prettyName+". Would " +
                    "you like to save it to another location?", // prompt
                "Yes", // default answer
                lastErrorSvc.getLastErrorMessage(), // text
                "Toolbox Error"); // title
            if (answer == "Yes") {
                while (1) {
                    var filename = filepicker_saveFile(
                            null, // default dir
                            uriparse_baseName(this.toolboxURL), // default filename
                            "Backup "+this.prettyName, // title
                            "Komodo Project", // default filter
                            ["Komodo Project", "All"]); // filters
                    if (!filename) return false;

                    this.toolbox.url = uriparse_localPathToURI(filename);
                    try {
                        this.toolbox.save();
                    } catch(ex) {
                        answer = dialog_yesNoCancel(
                            "There was an error saving the "+this.prettyName+
                                "to '"+filename+"'. Would you like to save "+
                                "it to different location?", // prompt
                            "Yes", // default answer
                            lastErrorSvc.getLastErrorMessage(), // text
                            "Toolbox Error"); // title
                        if (answer == "Yes") {
                            continue;
                        } else if (answer == "No") {
                            // do nothing
                        } else /* answer == "Cancel" */ {
                            return false;
                        }
                    }
                    break;
                }
            } else if (answer == "No") {
                // do nothing
            } else /* answer == "Cancel" */ {
                return false;
            }
        }
    }
    try {
        gObserverSvc.notifyObservers(this, 'file_changed', this.toolbox.url);
    } catch(e) { /* exception if no listeners */ }
    return true;
}



//---- Support for the user/personal Toolbox

function toolboxManager() {
    try {
        toolboxBaseManager.apply(this);
    } catch (e) {
        this.log.error("error creating toolboxManager: " + e);
    }
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
toolboxManager.prototype = new toolboxBaseManager();
toolboxManager.prototype.constructor = toolboxManager;

toolboxManager.prototype.init = function() {
    try {
        // Find out what our fname should be
        koDirs = Components.classes["@activestate.com/koDirs;1"].
                getService(Components.interfaces.koIDirs);
        os = Components.classes["@activestate.com/koOs;1"].
                getService(Components.interfaces.koIOs);
        var userDataDir = koDirs.userDataDir;
        var fname = os.path.join(userDataDir, 'toolbox.kpf');

        // Do the init
        this.name = "PersonalToolbox";
        var err = toolboxBaseManager.prototype._init.apply(this,
                    ["Toolbox", 'toolboxview', fname]);
        if (err == "could not create") {
            var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                               getService(Components.interfaces.koILastErrorService);
            dialog_internalError("Could not create a blank toolbox file.",
                                 lastErrorSvc.getLastErrorMessage());
        } else if (err == "could not load") {
            // create a backup and start with a blank one
            var badfname = fname+".error.txt";
            this.log.error("Unable to load '"+fname+"', "+
                           "a backup will be made at '"+badfname+"'.");
            alert("There was an error loading the Toolbox from '"+fname+"'. "+
                  "A backup will be made at '"+badfname+"' and a new "+
                  "toolbox will be created.");
            try {
                var shutil = Components.classes["@activestate.com/koShUtil;1"]
                                       .getService(Components.interfaces.koIShUtil);
                shutil.copyfile(fname, badfname);
            } catch (ex) {
                // Ignore it -- we can just go along w/ an empty toolbox (see below).
                alert("Could not backup toolbox data file: " +
                      this.lastErrorSvc.getLastErrorMessage());
            }
        }

        // Install sample toolbox for a particular version if necessary
        var sampletoolbox = os.path.join(koDirs.supportDir, 'samples');
        sampletoolbox = os.path.join(sampletoolbox, 'toolbox.kpf');
        var infoSvc = Components.classes["@activestate.com/koInfoService;1"].
            getService(Components.interfaces.koIInfoService);
        var prefName = "haveInstalledSampleToolbox-Komodo" + infoSvc.version;
        if (! gPrefs.hasBooleanPref(prefName) || ! gPrefs.getBooleanPref(prefName)) {
            this.log.info("Installing sample toolbox for version " + infoSvc.version);
            this.installSamples(sampletoolbox, infoSvc.version);
            gPrefs.setBooleanPref(prefName, "true");
        }
        this.loadFromProject(this.toolbox);
        // let the status service know it has work to do
        try {
            gObserverSvc.notifyObservers(this, 'file_update_now', null);
        } catch(e) { /* exception if no listeners */ }
    } catch (e) {
        log.exception(e);
    }
}

toolboxManager.prototype.installSamples = function(sampleToolboxPath, version) {
    gTimeline.enter('toolboxManager.installSamples');
    var sample_project = Components.classes["@activestate.com/koProject;1"]
                                .createInstance(Components.interfaces.koIProject);
    var folder = Components.classes["@activestate.com/koPart?type=folder;1"]
                                    .createInstance(Components.interfaces.koIPart_folder);

    sample_project.load(uriparse_localPathToURI(sampleToolboxPath));
    folder.name = "Samples (" + version + ")";
    this.log.info("installing samples: " + folder.name );
    var i;
    var children = new Array();
    sample_project.getChildren(children, new Object());
    children = children.value;
    for (i = 0; i < children.length; i++) {
        this.log.info("adding child " + children[i]);
        folder.addChild(children[i]);
    }
    this.toolbox.addChild(folder);
    gTimeline.leave('toolboxManager.installSamples');
}


//---- Support for the Shared Toolbox
//
// In a discussion btwn DavidA and TrentM it was decided that the shared
// toolbox behaviour should be the following (in light of the possibility
// that the current Komodo user might not have permissions to write to
// the shared toolbox location):
//  - If the useSharedToolbox pref is false, then shared toolbox support is
//    off.
//  - Else if the shared toolbox file exists then shared toolbox support is
//    on.
//  - Else if the user DOES have access to write to the shared toolbox file
//    location then a blank one is created (as for the normal Toolbox) and
//    shared toolbox support is on.
//  - Else (useSharedToolbox is true, the shared toolbox file does not
//    exist and the user does NOT have write access) shared toolbox support
//    is turned off.

// Update the shared toolbox state (i.e. enable or disable it).
function Toolbox_updateSharedToolbox() {
// #if PRODUCT_TYPE == "professional"
    try {
        // Abort early if there is nothing to update.
        var useSharedToolbox = gPrefs.getBooleanPref("useSharedToolbox");
        if (gSharedToolboxEnabled == null) { // this is first time in this method
            if (!useSharedToolbox) {
                // By default the Shared Toolbox is collapsed, so nothing
                // to do except note it.
                gSharedToolboxEnabled = false;
                return;
            }
        } else if (gSharedToolboxEnabled && useSharedToolbox) {
            // As long as the shared toolbox path is still the same, there
            // is nothing to do.
            var stPath = Toolbox_getSharedToolboxPath();
            var stURL = uriparse_localPathToURI(stPath);
            if (stURL == gSharedToolboxMgr.toolboxURL) {
                return;
            }
        } else if (!gSharedToolboxEnabled && !useSharedToolbox) {
            return;
        }

        // Try to apply the user's "Shared Toolbox" setting changes.
        if (gSharedToolboxMgr) {
            // Need to shutdown this existing shared toolbox manager first.
            gSharedToolboxMgr.close();
            for (index = 0; index < gPartManagers.length; ++index) {
                if (gPartManagers[index] == this) {
                    gPartManagers.splice(index, 1);
                    break;
                }
            }
            gSharedToolboxMgr = null;
        }
        var tabbox
        if (!useSharedToolbox) {
            // We're going to collapse the shared toolbox, but if it's
            // the selected tab, we should select the toolbox tab
            // instead.
            tabbox = document.getElementById('rightTabBox');
            if (tabbox.selectedTab == document.getElementById('sharedtoolbox_tab')) {
                tabbox.selectedTab = document.getElementById('toolbox_tab')
            }
            document.getElementById('sharedtoolboxviewbox')
                    .setAttribute('collapsed', 'true');
            document.getElementById('sharedtoolbox_tab')
                    .setAttribute('collapsed', 'true');
            document.getElementById('sharedtoolboxviewbox')
                    .setAttribute('hidden', 'true');
            document.getElementById('sharedtoolbox_tab')
                    .setAttribute('hidden', 'true');
            gSharedToolboxEnabled = false;
        } else {
            gSharedToolboxMgr = new sharedToolboxManager();
            var err = gSharedToolboxMgr.init();
            var _partSvc = Components.classes["@activestate.com/koPartService;1"]
                .getService(Components.interfaces.koIPartService);
            _partSvc.sharedToolbox = gSharedToolboxMgr.toolbox;
            if (err) {
                //XXX Could offer a button on this dialog to turn off
                //    Shared Toolbox support for now.
                dialog_alert(err);
                tabbox = document.getElementById('rightTabBox');
                if (tabbox.selectedTab == document.getElementById('sharedtoolbox_tab')) {
                    tabbox.selectedTab = document.getElementById('toolbox_tab')
                }
                document.getElementById('sharedtoolboxviewbox')
                        .setAttribute('collapsed', 'true');
                document.getElementById('sharedtoolbox_tab')
                        .setAttribute('collapsed', 'true');
                // <tabbox> Ctrl+Tab handling uses the 'hidden' attribute.
                document.getElementById('sharedtoolbox_tab')
                        .setAttribute('hidden', 'true');
                document.getElementById('sharedtoolbox_tab')
                        .setAttribute('hidden', 'true');
                gSharedToolboxEnabled = false;
            } else {
                document.getElementById('sharedtoolboxviewbox')
                        .removeAttribute('collapsed');
                document.getElementById('sharedtoolbox_tab')
                        .removeAttribute('collapsed');
                // <tabbox> Ctrl+Tab handling uses the 'hidden' attribute.
                document.getElementById('sharedtoolboxviewbox')
                        .removeAttribute('hidden');
                document.getElementById('sharedtoolbox_tab')
                        .removeAttribute('hidden');
                gSharedToolboxEnabled = true;
            }
        }
    } catch (e) {
        log.exception(e);
    }
// #endif
}


// #if PRODUCT_TYPE == "professional"

// Return the currently configured full path to the shared toolbox file.
function Toolbox_getSharedToolboxPath()
{
    try {
        var osPath = Components.classes["@activestate.com/koOsPath;1"].
                     getService(Components.interfaces.koIOsPath);
        var koDirs = Components.classes["@activestate.com/koDirs;1"].
                     getService(Components.interfaces.koIDirs);
        return osPath.join(koDirs.commonDataDir, "toolbox.kpf");
    } catch (ex) {
        log.exception(ex, "error determining shared toolbox path");
    }
    return null;
}


function sharedToolboxManager() {
    try {
        toolboxBaseManager.apply(this);
    } catch (ex) {
        log.exception(ex, "error creating sharedToolboxManager:");
    }
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
sharedToolboxManager.prototype = new toolboxBaseManager();
sharedToolboxManager.prototype.constructor = toolboxManager;

// This returns null if successful, otherwise it returns an error message.
sharedToolboxManager.prototype.init = function() {
    try {
        this.name = "SharedToolbox";
        var errmsg;
        var stPath = Toolbox_getSharedToolboxPath();
        var err = toolboxBaseManager.prototype._init.apply(this,
                    ["Shared Toolbox", "sharedtoolboxview", stPath]);
        var lastErrorSvc = Components.classes["@activestate.com/koLastErrorService;1"].
                           getService(Components.interfaces.koILastErrorService);
        if (err == "could not create") {
            errmsg = "You have enabled Shared Toolbox support but Komodo "+
                         "could not create a Shared Toolbox file at '"+stPath+
                         "': "+lastErrorSvc.getLastErrorMessage()+
                         ".  You must ensure that the file exists or "+
                         "can be created.";
            return errmsg;
        } else if (err == "could not load") {
            errmsg = "Could not load the Shared Toolbox file at '"+stPath+
                         "': "+lastErrorSvc.getLastErrorMessage();
            return errmsg;
        }
    } catch (e) {
        log.exception(e);
    }
    return null;
}


// A pref observer to watch if the user turns the shared toolbox on or off.
function _ToolboxPrefObserver() {};
_ToolboxPrefObserver.prototype.observe = function(prefSet, prefSetID, prefName)
{
    Toolbox_updateSharedToolbox();
};

_ToolboxPrefObserver.prototype.init = function() {
    gPrefs.prefObserverService.addObserver(this, "useSharedToolbox", false);
    gPrefs.prefObserverService.addObserver(this, "commonDataDirMethod", false);
    gPrefs.prefObserverService.addObserver(this, "customCommonDataDir", false);
}

_ToolboxPrefObserver.prototype.destructor = function() {
    if (gPrefSvc && gPrefs) {
        gPrefs.prefObserverService.removeObserver(this, "useSharedToolbox");
        gPrefs.prefObserverService.removeObserver(this, "commonDataDirMethod");
        gPrefs.prefObserverService.removeObserver(this, "customCommonDataDir");
    }
}
// #endif


function toolbox_toolboxController() {
}

// The following two lines ensure proper inheritance (see Flanagan, p. 144).
toolbox_toolboxController.prototype = new Controller();
toolbox_toolboxController.prototype.constructor = toolbox_toolboxController;

toolbox_toolboxController.prototype.destructor = function() {
}

toolbox_toolboxController.prototype.is_cmd_toolboxExportPackage_enabled = function () {
    try {
        return (gFocusedProjectView && gFocusedProjectView.getSelectedItems().length > 0);
    } catch (e) {
        log.exception(e);
    }
    return false;
}

toolbox_toolboxController.prototype.do_cmd_toolboxExportPackage = function () {
    if (gFocusedProjectView) {
        Toolbox_ExportPackageItems(gFocusedProjectView.getSelectedItems());
    }
}

