"""Language package for HTML"""

from xpcom import components, ServerException

from koLanguageServiceBase import *

def registerLanguage(registery):
    registery.registerLanguage(koHTMLLanguage())

class koHTMLLanguage(KoLanguageBase):
    name = "HTML"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{7FDF82EB-7E8E-40f4-BD39-563FD2B7E94A}"

    accessKey = 'm'
    primary = 1
    defaultExtension = ".html" # XXX how do we deal w/ .htm vs. html?
    #XXX Boy it would be nice to have this be 6 so we could get green
    #    squigglies for linting.
    styleBits = 7
    commentDelimiterInfo = { "block": [ ("<!--", "-->") ]  }
    indicatorBits = 1
    supportsSmartIndent = "XML"
    _lineup_chars = '' # don't indent ()'s and the like in HTML!
    _lineup_open_chars = "" # don't indent ()'s and the like in HTML!
    sample = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
	"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en" dir="ltr">
  <head lang="en" xml:lang="en" dir="ltr" profile="profile">
    <!-- base href="http://www.w3.org/People/mimasa/test/xhtml/media-types/test" /-->
    <meta http-equiv="Content-Language" content="en" />
    <title lang="en" xml:lang="en" dir="ltr">Sample XHTML 1.0 document</title>
    <link rev="made" href="mailto:mimasa@w3.org" />
    <style type="text/css" media="screen">
      .title { color: #FF3333 }
    </style>
    <meta scheme="Dublin Core" name="DC.CREATOR" content="ISHIKAWA, Masayasu" />
    <style type="text/css" media="aural">
      .title { volume: loud }
    </style>
    <link id="help.link" class="help.link" title="For more information, see HTML 4 Specification." lang="en" xml:lang="en" dir="ltr" charset="ISO-8859-1" href="http://www.w3.org/TR/html4" rel="Help" />
  </head>
  <body id="body" class="body" title="body" lang="en" xml:lang="en" dir="ltr">

    <h1 id="title" class="title" title="document title" lang="en"
    xml:lang="en" dir="ltr" style="color: maroon">Sample XHTML 1.0 document</h1>

    <h2 id="ToC">Contents</h2>

    <ul>
      <li><a href="#grouping.heading">Grouping elements: <code>div</code> and <code>span</code></a></li>
      <li><a href="#bidi.heading">Bidrectionality</a>
        <ul>
	  <li><a href="#ltr.heading">Base directionality: left to right</a></li>
	  <li><a href="#rtl.heading">Base directionality: right to left</a></li>
	  <li><a href="#bdo.heading">Bidirectionality override: <code>bdo</code></a></li>
	</ul>
      </li>

"""
    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_HTML)
            # XXX config options needed?
            self._lexer.setProperty('fold.html','0')
            #self._lexer.setProperty('fold.compact','1')
            self._lexer.supportsFolding = 0
            # keyword sets
            # 0 = html # 1 = JavaScript # 2 = VBScript # 3 = PythonScript # 4 = PHP
            self._lexer.setKeywords(1, self._js_keywords)
            self._lexer.setKeywords(2, self._vb_keywords)
            self._lexer.setKeywords(4, self._php_keywords)
        return self._lexer


    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService("@activestate.com/koHTMLLinter;1")
        return self._linter

    # XXX later these need to be centralized. koPHPLanguage.py has the same
    _php_keywords = [
                # new to php5
                "public", "private", "protected", "final",
                "abstract", "interface", "implements",
                "try", "catch", "throw", "instanceof",
                # existed in php4
                "define", "true", "false", 
                "int", "integer", "real", "double",
                "float", "string",  "object", "bool", "boolean",
                "this", "virtual", "parent", "null",
                # http://www.php.net/manual/en/reserved.php#reserved.keywords
                "and", "or", "xor", "__file__", "__line__",
                "array", "as", "break", "case", "cfunction",
                "class", "const", "continue", "declare"
                "default", "die", "do", "echo", "else",
                "elseif", "empty", "enddeclare", "endfor", "endforeach",
                "endif", "endswitch", "endwhile", "eval", "exit",
                "extends", "for", "foreach", "function", "global",
                "if", "include", "include_once", "isset", "list",
                "new", "old_function", "print", "require", "require_once",
                "return", "static", "switch", "unset", "use",
                "var", "while", "_function_", "_class_",
                # http://www.php.net/manual/en/reserved.constants.core.php
                "php_version",
                "php_os",
                "default_include_path",
                "pear_install_dir",
                "pear_extension_dir",
                "php_extension_dir",
                "php_bindir",
                "php_libdir",
                "php_datadir",
                "php_sysconfdir",
                "php_localstatedir",
                "php_config_file_path",
                "php_output_handler_start",
                "php_output_handler_cont",
                "php_output_handler_end",
                "e_error",
                "e_warning",
                "e_parse",
                "e_notice",
                "e_core_error",
                "e_core_warning",
                "e_compile_error",
                "e_compile_warning",
                "e_user_error",
                "e_user_warning",
                "e_user_notice",
                "e_all",
                # http://www.php.net/manual/en/reserved.constants.standard.php
                "extr_overwrite",
                "extr_skip",
                "extr_prefix_same",
                "extr_prefix_all",
                "extr_prefix_invalid",
                "extr_prefix_if_exists",
                "extr_if_exists",
                "sort_asc",
                "sort_desc",
                "sort_regular",
                "sort_numeric",
                "sort_string",
                "case_lower",
                "case_upper",
                "count_normal",
                "count_recursive",
                "assert_active",
                "assert_callback",
                "assert_bail",
                "assert_warning",
                "assert_quiet_eval",
                "connection_aborted",
                "connection_normal",
                "connection_timeout",
                "ini_user",
                "ini_perdir",
                "ini_system",
                "ini_all",
                "m_e",
                "m_log2e",
                "m_log10e",
                "m_ln2",
                "m_ln10",
                "m_pi",
                "m_pi_2",
                "m_pi_4",
                "m_1_pi",
                "m_2_pi",
                "m_2_sqrtpi",
                "m_sqrt2",
                "m_sqrt1_2",
                "crypt_salt_length",
                "crypt_std_des",
                "crypt_ext_des",
                "crypt_md5",
                "crypt_blowfish",
                "directory_separator",
                "seek_set",
                "seek_cur",
                "seek_end",
                "lock_sh",
                "lock_ex",
                "lock_un",
                "lock_nb",
                "html_specialchars",
                "html_entities",
                "ent_compat",
                "ent_quotes",
                "ent_noquotes",
                "info_general",
                "info_credits",
                "info_configuration",
                "info_modules",
                "info_environment",
                "info_variables",
                "info_license",
                "info_all",
                "credits_group",
                "credits_general",
                "credits_sapi",
                "credits_modules",
                "credits_docs",
                "credits_fullpage",
                "credits_qa",
                "credits_all",
                "str_pad_left",
                "str_pad_right",
                "str_pad_both",
                "pathinfo_dirname",
                "pathinfo_basename",
                "pathinfo_extension",
                "char_max",
                "lc_ctype",
                "lc_numeric",
                "lc_time",
                "lc_collate",
                "lc_monetary",
                "lc_all",
                "lc_messages",
                "abday_1",
                "abday_2",
                "abday_3",
                "abday_4",
                "abday_5",
                "abday_6",
                "abday_7",
                "day_1",
                "day_2",
                "day_3",
                "day_4",
                "day_5",
                "day_6",
                "day_7",
                "abmon_1",
                "abmon_2",
                "abmon_3",
                "abmon_4",
                "abmon_5",
                "abmon_6",
                "abmon_7",
                "abmon_8",
                "abmon_9",
                "abmon_10",
                "abmon_11",
                "abmon_12",
                "mon_1",
                "mon_2",
                "mon_3",
                "mon_4",
                "mon_5",
                "mon_6",
                "mon_7",
                "mon_8",
                "mon_9",
                "mon_10",
                "mon_11",
                "mon_12",
                "am_str",
                "pm_str",
                "d_t_fmt",
                "d_fmt",
                "t_fmt",
                "t_fmt_ampm",
                "era",
                "era_year",
                "era_d_t_fmt",
                "era_d_fmt",
                "era_t_fmt",
                "alt_digits",
                "int_curr_symbol",
                "currency_symbol",
                "crncystr",
                "mon_decimal_point",
                "mon_thousands_sep",
                "mon_grouping",
                "positive_sign",
                "negative_sign",
                "int_frac_digits",
                "frac_digits",
                "p_cs_precedes",
                "p_sep_by_space",
                "n_cs_precedes",
                "n_sep_by_space",
                "p_sign_posn",
                "n_sign_posn",
                "decimal_point",
                "radixchar",
                "thousands_sep",
                "thousep",
                "grouping",
                "yesexpr",
                "noexpr",
                "yesstr",
                "nostr",
                "codeset",
                "log_emerg",
                "log_alert",
                "log_crit",
                "log_err",
                "log_warning",
                "log_notice",
                "log_info",
                "log_debug",
                "log_kern",
                "log_user",
                "log_mail",
                "log_daemon",
                "log_auth",
                "log_syslog",
                "log_lpr",
                "log_news",
                "log_uucp",
                "log_cron",
                "log_authpriv",
                "log_local0",
                "log_local1",
                "log_local2",
                "log_local3",
                "log_local4",
                "log_local5",
                "log_local6",
                "log_local7",
                "log_pid",
                "log_cons",
                "log_odelay",
                "log_ndelay",
                "log_nowait",
                "log_perror"
                ]

    _js_keywords = ["abstract", "boolean", "break", "byte", "case", "catch",
                 "char", "class", "const", "continue", "debugger", "default",
                 "delete", "do", "double", "else", "enum", "export",
                 "extends", "final", "finally", "float", "for", "function",
                 "goto", "if", "implements", "import", "in", "instanceof",
                 "int", "interface", "long", "native", "new", "package",
                 "private", "protected", "public", "return", "short",
                 "static", "super", "switch", "synchronized", "this", "throw",
                 "throws", "transient", "try", "typeof", "var", "void",
                 "volatile", "while", "with"]

    _vb_keywords = ["and", "as", "attribute", "begin", 
                    "byref", "byval", "case", "call", "const",
                    "continue", "dim", "do", "each", "else",
                    "elseif", "end", "error", "exit", "false",
                    "for", "function", "global", "goto", "if",
                    "in", "loop", "me", "new", "next", "not",
                    "nothing", "on", "optional", "or", "private",
                    "public", "redim", "rem", "resume", "select",
                    "set", "sub", "then", "to", "true", "type",
                    "while", "with", "boolean", "byte", "currency",
                    "date", "double", "integer", "long", "object",
                    "option", "single", "string", "type", "variant"]
