#!python
# Copyright (c) 2000-2006 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.


from xpcom import components
from xpcom.server.enumerator import *
from koLanguageServiceBase import KoCompletionLanguageService
import os, sys, traceback, re
import tempfile
import string
import threading, thread
import time
import uriparse
from codeintel.phpcile import *

iface = components.interfaces.koICodeIntelCompletionUIHandler

import logging
log = logging.getLogger("phpAutocompletion")
#log.setLevel(logging.DEBUG)

wordbreaks = "_" + string.uppercase + string.lowercase + string.digits

# > is not the best solution, -> is the actual deliminator for
# objects
variablespec  = '$.>"'

global_spec = "_global_"

#config observer class to update php config info

class KoPHPCompletion(KoCompletionLanguageService):
    _com_interfaces_ = [components.interfaces.koICompletionLanguageService,
                        components.interfaces.nsIObserver]
    _reg_desc_ = "PHP Calltip/AutoCompletion Service"
    _reg_clsid_ = "{9BC17B87-140A-4da4-8730-A6EC9F6953E7}"
    _reg_contractid_ = "@activestate.com/koPHPCompletionLanguageService;1"
    
    _globalCile = None
    php_config = None
    _cileCache = None
    
    def __init__(self):
        #log.debug('KoPHPCompletion.__init__')
        # set up config observer
        if not self.php_config:
            self.php_config = components.classes["@activestate.com/koPHPInfoEx;1"].getService(components.interfaces.koIPHPInfoEx)

        self.sysUtils = components.classes["@activestate.com/koSysUtils;1"].\
            getService(components.interfaces.koISysUtils)

        self.koDirs = components.classes["@activestate.com/koDirs;1"].\
              getService(components.interfaces.koIDirs)
            
        self.triggers = ''
        self.triggersCallTip = '(),'
        self._braceCount = 0
        self._scintilla = None
        self._functext = ''
        self._ok = 0
        self._lastfuncnames = []
        self._lastcompletion = None
        self.currentFile = ""
        self.currentFileText = None
        self.doInclude = 0
        self._lastscan = -1
        self.scanLock = threading.Lock()

        # we setup our autocomplete cache now
        if not KoPHPCompletion._cileCache:
            KoPHPCompletion._cileCache = PHPcile()

        if not self._globalCile:
            self._getFuncTipsFromFile()

        self.prefService = None

    def __del__(self):
        if self.prefService:
            self.prefService.prefs.removeObserver(self)
        
    def observe(self, subject, topic, data):
        if topic == 'phpDefaultInterpreter':
            self._getFuncTipsFromFile()

    def _getFuncTipsPath(self, version):
        """Return the path to the best PHP functips file for this
        version of PHP.
        """
        functip_dname = os.path.join(self.koDirs.supportDir, "php")
        if version:
            ver_tuple = tuple(map(int, version.split('.')[:2]))
            path = os.path.join(functip_dname,
                "phpfunctips-%s.%s.txt" % ver_tuple)
            if os.path.exists(path):
                return path
            
        # Don't have a phpfunctips file matching this PHP version. We
        # need to gather a list of phpfunctips that we *do* have.
        available_vers = []
        pattern = os.path.join(functip_dname, "phpfunctips-*.*.txt")
        for fpath in glob.glob(pattern):
            fname = os.path.basename(fpath)
            v_string = fname[len("phpfunctips-"):-len(".txt")]
            try:
                v = tuple(map(int, v_string.split('.')[:2]))
            except ValueError, ex:
                continue # skip if the version bits aren't integers
            available_vers.append(v)
        available_vers.sort()
        #print "PHP functips path: available_vers=%r" % available_vers

        if version:
            # Look for the closest preceding version in the available
            # phpfunctip files.
            import bisect
            # "-1" because we want to fallback to preceding version.
            # "max(0, ...)" because we clip at the earliest version we know
            # about.
            index = max(0, bisect.bisect(available_vers, ver_tuple)-1)
            #print "PHP functips path: ver=%r -> index=%r" % (ver_tuple, index)
            functip_ver = available_vers[index]
        else:
            # Use the latest phpfunctips version.
            functip_ver = available_vers[-1] 

        path = os.path.join(functip_dname,
                            "phpfunctips-%s.%s.txt" % functip_ver)
        #print "PHP functips path: version=%r -> '%s'" % (version, path)
        return path
    
    def _getFuncTipsFromFile(self):
        #log.debug('KoPHPCompletion._getFuncTipsFromFile')
        ver = self.php_config.version
        #log.debug('KoPHPCompletion._getFuncTipsFromFile php version %s', ver)
        self._fname = self._getFuncTipsPath(ver)
        #log.debug('KoPHPCompletion._getFuncTipsFromFile: %s', self._fname)
        threading.Thread(target = self._background, args=()).start()
        
    def _background(self, *args):
        #log.debug('KoPHPCompletion._background')
        self.load_funcs()
        
    def load_funcs(self):
        #log.debug('KoPHPCompletion.load_funcs')
        # read PHP function definitions
        try:
            inputfile = open(self._fname,"r")
            filedata = inputfile.read()
            inputfile.close()
            i=0
            #theRe = re.compile(r'^(?:(?P<attributes>public|private|protected)\s*)?(?:(?P<type>[\w_\[\]\|]*?)\s*)?(?P<classname>[\w_]*?)(?:::)?(?P<name>[\w_]*?)\s*?\((?P<args>.*?)\)(?:\s+throws\s+(?P<throws>[\w_]*?))?')
            theRe = re.compile(r'^(?:(?P<attributes>public\s+|private\s+|protected\s+|static\s+)*)(?:(?P<type>[\w_\[\]\|]*?)\s*)?(?P<classname>[\w_]*?)(?:::)?(?P<name>[\w_]+?)\s*?\((?P<args>.*?)\)(?:\s+throws\s+(?P<throws>[\w_]+))?;?',re.M|re.S)
            
            # remove comments from file data
            comments = re.compile(r'^#.*$',re.M)
            filedata = comments.sub('',filedata)
            
            self._globalCile = PHPFile(global_spec)
            currentFunc = None
            startPos = 0

            while 1:
                found = theRe.search(filedata, startPos)
                if not found: break
                
                if currentFunc:
                    currentFunc.doc = filedata[startPos:found.start(0)].strip()
                
                startPos = found.end(0)
                match = found.groupdict()

                funclist = self._globalCile.functions
                classname = ''
                if 'classname' in match and match['classname']:
                    # find or create the class object
                    classname = match['classname']
                    if classname not in self._globalCile.classes:
                        #log.debug("adding class %s", classname)
                        self._globalCile.classes[classname] = PHPClass(classname,
                                                                  '', 0, 0)
                    funclist = self._globalCile.classes[classname].functions
                # add the new function the the funclist
                attributes = ''
                if 'attributes' in match and match['attributes']:
                    attributes=match['attributes']
                currentFunc = PHPFunction(match['name'], match['args'], 0,
                                    attributes=attributes, classname=classname)
                if 'type' in match:
                    currentFunc.returnType = match['type']
                funclist[match['name']] = currentFunc
        except:            #log.debug("PHP NumFunctions: %d",len(self._globalCile.functions))
            #log.debug("PHP num classes: %d",len(self._globalCile.classes))

            log.exception("PHP Failed loading function list")
            pass
        self._ok = 1

    def _scanFileCompletion(self):
        #log.debug('KoPHPCompletion._scanFileCompletion')
        if not self.scanLock.acquire(0): return
        lex = PHPLexer(self.currentFile,
                       content=self.currentFileText,
                       include=self.doInclude,
                       include_path=self.php_config.include_path,
                       cache=self._cileCache)
        self.currentFileText = ""
        self.scanLock.release()
        
    def scanBufferCompletion(self, scimoz, filename):
        #log.debug('KoPHPCompletion.scanBufferCompletion')
        # prevent multiple scans at the same time
        self._scintilla = scimoz
        lastfile = self.currentFile
        if filename:
            # get local file system path so we can include files correctly
            try:
                self.currentFile = uriparse.URIToLocalPath(filename)
            except:
                # not a local file!  Lets parse it anyway, just includes
                # will not work correctly for this file
                pass
            
            # allow filename to be used anyway, this allows remote files
            # to be correctly parsed, we only do this on parsing the buffer
            if not self.currentFile:
                self.currentFile = filename
        else:
            # XXX ok for beta, need to deal with multiple new files
            self.currentFile = "__newfile__"

        self.doInclude = 1
        # only do this if we actually have something to parse
        if scimoz.textLength > 0:
            self.currentFileText = scimoz.text
            self._lastfuncnames = []
            self._lastcompletion = None
            threading.Thread(target = self._scanFileCompletion, args=()).start()
    
    def _getClassTipArgs(self, funcname, classname):
        #log.debug('KoPHPCompletion._getClassTipArgs')
        cl = None
        if self._cileCache.classindex.has_key(classname):
            _file = self._cileCache.filesparsed[self._cileCache.classindex[classname]]
            cl = _file.classes[classname]
        elif classname in self._globalCile.classes:
            cl = self._globalCile.classes[classname]
            
        if cl:
            if cl.functions.has_key(funcname):
                return "%s"%cl.functions[funcname]
            # check parent classes
            nest = 0
            while cl.extends and self._cileCache.classindex.has_key(cl.extends):
                _file = self._cileCache.filesparsed[self._cileCache.classindex[cl.extends]]
                cl = _file.classes[cl.extends]
                if cl.functions.has_key(funcname):
                    return "%s"%cl.functions[funcname]
                # protect against endless loop
                nest = nest + 1
                if nest > 3: break
        return None
        
    # get the function arguments
    def gettipargs(self, funcname, classname):
        #log.debug('KoPHPCompletion.gettipargs')
        # check the global namespace
        tipargs = None
        if not classname:
            # check the global built in php functions
            if self._globalCile.functions.has_key(funcname):
                tipargs = "%s"%self._globalCile.functions[funcname]
            # check the user namespace
            elif self._cileCache.functionindex.has_key(funcname):
                _file = self._cileCache.filesparsed[self._cileCache.functionindex[funcname]]
                if _file.functions.has_key(funcname):
                    tipargs = "%s"%_file.functions[funcname]
            # check for class constructors
            else:
                tipargs = self._getClassTipArgs(funcname, funcname)
                if not tipargs:
                    tipargs = self._getClassTipArgs('__construct', funcname)
        else:
            tipargs = self._getClassTipArgs(funcname, classname)
        if tipargs and len(tipargs) > 30:
            # changes a call tip looking like (x [, y])
            # to look like: (x, [y])
            # so that in the end it looks like:
            # (x,
            #  [y])
            tipargs = tipargs.replace('[, ',',[')
            tipargs = tipargs.replace(',',",\n    ")
        return tipargs

    def _combineItems(self, items, inc_items):
        #log.debug('KoPHPCompletion._combineItems')
        if len(items):
            for item in inc_items:
                #print "removing ",repr(item)
                try:
                    items.remove(item)
                except ValueError, e:
                    pass
        items.extend(inc_items)
        return items
    
    def _getFunctionScope(self, filename, line):
        #log.debug('KoPHPCompletion._getFunctionScope line %d file %s', line, filename)
        if self._cileCache.filesparsed.has_key(filename):
            _file = self._cileCache.filesparsed[filename]
            for fn_name, fn in _file.functions.items():
                #log.debug("%s? l:%d s:%d e:%d", fn_name, line, fn.line, fn.lineend)
                if fn.line <= line and fn.lineend >= line:
                    return fn
            for cn_name, cn in _file.classes.items():
                #log.debug("%s? l:%d s:%d e:%d", cn_name, line, cn.linestart, cn.lineend)
                if cn.linestart <= line and cn.lineend >= line:
                    for fn_name, fn in cn.functions.items():
                        #log.debug("%s? l:%d s:%d e:%d", fn_name, line, fn.line, fn.lineend)
                        if fn.line <= line and fn.lineend >= line:
                            return fn
        return None
        
    def _getvariables(self, filename, name, curline):
        #log.debug('KoPHPCompletion._getvariables startwith %s line %d file %s', name, curline, filename)
        # first, recurse the included files
        # import the superglobals from _getpredefined
        items = self._getpredefined(name)
        scope = self._getFunctionScope(filename, curline)
        if scope:
            # get variables from function
            varnames = {}
            varnames.update(scope.variables)
            varnames.update(scope.args.args)
            inc_items = [key for key in varnames.keys() if key.lower().startswith(name.lower())]
            items = self._combineItems(items, inc_items)
            return items
        
        if self._cileCache.needfile.has_key(filename):
            for file in self._cileCache.needfile[filename]:
                inc_items = self._getvariables(file, name, curline)
                #print "found %d vars in %s"%(len(inc_items),file)
                items = self._combineItems(items, inc_items)
        
        if self._cileCache.filesparsed.has_key(filename):
            _file = self._cileCache.filesparsed[filename]
            varnames = _file.variables.keys()
            inc_items = [key for key in varnames if key.lower().startswith(name.lower()) \
                         and (filename != self.currentFile or _file.variables[key].line <= curline)]
            items = self._combineItems(items, inc_items)
        return items

    def _getpredefined(self, name):
        """ optionally add superglobals to the list  """
        if name[0] == '_': # looking for builtins like $_GET
            items = [ # special case, hard coded. Manly!
                '_COOKIE',
                '_ENV',
                '_FILES',
                '_GET',
                '_POST',
                '_REQUEST',
                '_SERVER',
                '_SESSION',
                ]
        elif name[:2] == 'HT': # for the old $HTTP_* super globals
            items = [
                'HTTP_COOKIE_VARS',
                'HTTP_ENV_VARS',
                'HTTP_GET_VARS',
                'HTTP_POST_FILES',
                'HTTP_POST_VARS',
                'HTTP_SERVER_VARS',
                'HTTP_SESSION_VARS',
                ]
        elif name[:2] == 'GL': # for $GLOBALS
            items = ['GLOBALS']
        else:
            items = []
        return items


    def getvariables(self, name, curline):
        #log.debug('KoPHPCompletion.getvariables that start with %s', name)
        items = ["%s?%s" % (i, iface.ACIID_VARIABLE) for i in 
                self._getvariables(self.currentFile, name, curline)
                ]
        items.sort()
        return chr(self.completionSeparator).join(items), 1

    def getClassForClassName(self, classname):
        cl = None
        if self._cileCache.classindex.has_key(classname):
            fn = self._cileCache.classindex[classname];
            if self._cileCache.filesparsed.has_key(fn):
                _file = self._cileCache.filesparsed[fn]
                if _file.classes.has_key(classname):
                    cl = _file.classes[classname]
        elif classname in self._globalCile.classes:
            cl = self._globalCile.classes[classname]
        return cl
        
    # given a linenumber, returns the classname of the parent class
    # at that line.  currentclass is always in currentfile!
    def getParentClassName(self, lineno):
        classname = self.getCurrentClassName(lineno)
        if classname:
            cl = self.getClassForClassName(classname)
            if cl.extends:
                return cl.extends
        return None
    
    # given a linenumber, returns the classname at that line
    #currentclass is always in currentfile!
    def getCurrentClassName(self, lineno):
        #log.debug('KoPHPCompletion.getCurrentClassName in %s at line %d', self.currentFile, lineno)
        if self._cileCache.filesparsed.has_key(self.currentFile):
            _file = self._cileCache.filesparsed[self.currentFile]
            classes = _file.classes.keys()
            for classname in classes:
                if lineno > _file.classes[classname].linestart and \
                   lineno < _file.classes[classname].lineend:
                    return classname
        return None

    # given a classname, returns the members
    def _getClassMembers(self, classname):
        #log.debug('KoPHPCompletion._getClassMembers %r', classname)
        items = []
        cl = self.getClassForClassName(classname)
        if cl:
            #log.debug("   class is in file %s",fn)
            items = ['%s?%s' %(k, iface.ACIID_FUNCTION) for k in cl.functions.keys()]
            members = ['%s?%s' %(k, iface.ACIID_VARIABLE) for k in cl.members.keys()]
            items.extend(members)
            # if we subclassed, get parents info also
            if cl.extends:
                pitems = self._getClassMembers(cl.extends)
                # remove overridden functions
                items = self._combineItems(pitems, items)
        return items

    def getClassMembers(self, classname):
        #log.debug('KoPHPCompletion.getClassMembers %s', classname)
        items = self._getClassMembers(classname)
        items.sort()
        return chr(self.completionSeparator).join(items), 1
    
    # given a variable name, returns the class name
    def getClassName(self, varname, filename=None):
        #log.debug('KoPHPCompletion.getClassName from class var %s', varname)
        if not filename: filename = self.currentFile
        if self._cileCache.filesparsed.has_key(filename):
            _file = self._cileCache.filesparsed[filename]
            if _file.variables.has_key(varname):
                # get the classname, exit if not a class variable
                return _file.variables[varname].types[0][1]
            #look in included files to find variable
            if self._cileCache.needfile.has_key(filename):
                for fn in self._cileCache.needfile[filename]:
                    cn = self.getClassName(varname,fn)
                    if cn: return cn
        return None


    def _getclasses(self, filename, depth=0):
        #log.debug('KoPHPCompletion._getclasses in file %s', filename)
        # first, recurse the included files.  we limit the recurse depth to 10
        items = []
        if depth > 10 or not self._cileCache.filesparsed.has_key(filename):
            return items
        
        _file = self._cileCache.filesparsed[filename]
        
        for inc_filname in _file.includes.keys():
            inc_items = self._getclasses(inc_filname, depth+1)
            if inc_items:
                items = self._combineItems(items, inc_items)
        
        inc_items = _file.classes.keys()
        if inc_items:
            items = self._combineItems(items, inc_items)

        return items
    
    # get a sorted list of the classes available
    def getclasses(self):
        #log.debug('KoPHPCompletion.getclasses')
        items = self._getclasses(self.currentFile)
        # now add the global classes
        items += self._globalCile.classes.keys()
        items = ["%s?%s" % (i, iface.ACIID_CLASS) for i in items]
        items.sort()
        return chr(self.completionSeparator).join(items), 1

    def _getinterfaces(self, filename, depth=0):
        #log.debug('KoPHPCompletion._getclasses in file %s', filename)
        # first, recurse the included files.  we limit the recurse depth to 10
        items = []
        if depth > 10 or not self._cileCache.filesparsed.has_key(filename):
            return items
        
        _file = self._cileCache.filesparsed[filename]
        
        for inc_filname in _file.includes.keys():
            inc_items = self._getinterfaces(inc_filname, depth+1)
            if inc_items:
                items = self._combineItems(items, inc_items)
        
        inc_items = _file.interfaces.keys()
        if inc_items:
            items = self._combineItems(items, inc_items)

        return items
    
    # get a sorted list of the classes available
    def getinterfaces(self):
        #log.debug('KoPHPCompletion.getclasses')
        items = self._getinterfaces(self.currentFile)
        # now add the global classes
        items += self._globalCile.interfaces.keys()
        items = ["%s?%s" % (i, iface.ACIID_INTERFACE) for i in items]
        items.sort()
        return chr(self.completionSeparator).join(items), 1
    

    def _getfunctions(self, filename):
        #log.debug('KoPHPCompletion._getfunctions')
        # first, recurse the included files
        items = []
        if self._cileCache.needfile.has_key(filename):
            for file in self._cileCache.needfile[filename]:
                inc_items = self._getfunctions(file)
                items = self._combineItems(items, inc_items)
        
        if self._cileCache.filesparsed.has_key(filename):
            _file = self._cileCache.filesparsed[filename]
            inc_items = _file.functions.keys()
            items = self._combineItems(items, inc_items)
        return items
    
    # get the available completions
    def getmethods(self, name):
        #log.debug('KoPHPCompletion.getmethods')
        items = []
        if self._lastcompletion is not None and \
            self._lastcompletion == name[:len(self._lastcompletion)]:
            items = [key for key in self._lastfuncnames if key.lower().startswith(name.lower())]

            #if same list, skip doing it.
            if self._scintilla.autoCActive() and \
               (len(items) == len(self._lastfuncnames)): 
                #print "PHP keeping current autocomplete"
                return None, 0
        
        if (len(items)==0):
            funcnames = self._getfunctions(self.currentFile)
            _functions = [key for key in funcnames if key.lower().startswith(name.lower())]
            funcnames = self._globalCile.functions.keys()
            items = [key for key in funcnames if key.lower().startswith(name.lower())]
            items.extend(_functions)
            items.sort()

        if len(items):
            self._lastcompletion = name
            self._lastfuncnames = items
        else:
            self._lastcompletion = None
            self._lastfuncnames = []

        items = ["%s?%s" % (i, iface.ACIID_FUNCTION) for i in items]
        return chr(self.completionSeparator).join(items), 1

    def StartCallTip(self, ch, scimoz):
        #log.debug('KoPHPCompletion.StartCallTip')
        if not self._ok: return
        s = self._scintilla = scimoz
        selStart, selEnd = s.selectionStart, s.selectionEnd
        if selEnd == selStart and selStart > 0:
            style = s.getStyleAt(s.positionBefore(selStart))
            if style == s.SCE_HPHP_COMMENT or style == s.SCE_HPHP_COMMENTLINE:
                return
            
            if s.callTipActive():
                if ch == ')':
                    self._braceCount = self._braceCount - 1
                    if (self._braceCount < 1):
                        s.callTipCancel()
                elif ch == '(':
                    self._braceCount = self._braceCount + 1
                else:
                    self._ContinueCallTip()
            elif s.autoCActive():
                if ch == '(':
                    self._braceCount = self._braceCount + 1
                    self._StartCallTip()
                elif ch == ')':
                    self._braceCount = self._braceCount - 1
            else:
                if ch == '(':
                    self._braceCount = 1
                    self._StartCallTip()

    def AutoComplete(self, ch, scimoz):
        #log.debug('KoPHPCompletion.AutoComplete')
        if not self._ok or scimoz.autoCActive(): return
        if not self.prefService:
            # setup an observer on prefs so we can watch for interpreter
            # changes, and update our function tip list correctly.
            self.prefService = components.classes["@activestate.com/koPrefService;1"].getService(components.interfaces.koIPrefService);
            self.prefService.prefs.prefObserverService.addObserver(self, 'phpDefaultInterpreter',0);

        self._scintilla = scimoz
        posBefore = scimoz.positionBefore(scimoz.currentPos)
        style = scimoz.getStyleAt(posBefore) & 127
        #log.debug("keyPressed style: %r char: %r ", style, scimoz.text[scimoz.currentPos-1])
        prevstyle = scimoz.getStyleAt(scimoz.currentPos-2) & 127
        #log.debug("keyPressed prevstyle: %r char: %r", prevstyle, scimoz.text[scimoz.currentPos-2])
        if style == scimoz.SCE_HPHP_DEFAULT:
            self._StartAutoComplete()
        elif style == scimoz.SCE_HPHP_VARIABLE or \
            style == scimoz.SCE_HPHP_HSTRING_VARIABLE:
            self._VariableAutoComplete()
        else:
            char = scimoz.getWCharAt(posBefore)
            #log.debug("keyPressed wchar: %r ", char)
            if (char == ">" or char == ":") and \
                (style == scimoz.SCE_HPHP_OPERATOR or \
                (scimoz.currentPos > 3 and scimoz.getStyleAt(scimoz.positionAtChar(scimoz.currentPos,-3)) & 127 == scimoz.SCE_HPHP_HSTRING_VARIABLE)):
                pchar = scimoz.getWCharAt(scimoz.positionBefore(posBefore))
                if pchar == "-":
                    # class completion!  varname->function
                    self._ClassAutoComplete()
                elif pchar == ":":
                    # class completion!  classname::function
                    self._ClassAutoComplete(1)

    def _ClassAutoComplete(self, explicit = 0):
        #log.debug('KoPHPCompletion._ClassAutoComplete')
        # autocomplete works with _entire_ buffer as it scans it for attributes.
        s = self._scintilla
        byte_linelen, lastline = s.getCurLine()
        linelen = len(lastline)
        curpos = s.selectionStart
        curline = s.lineFromPosition(curpos)
        bytepos = curpos - s.positionFromLine(curline)-2
        pos = self.sysUtils.charIndexFromPosition(lastline, bytepos)
        if pos <= 0 or pos > linelen: return
        text = lastline[:pos]
        text = self._GetPrevWordFromText(text)
        #log.debug("    PHP word is: [%s]", text)

        if text in ["this", "self"]:
            # find the current class name
            text = self.getCurrentClassName(curline)
            #log.debug("    PHP classname: [%s]", text)
        elif text == "parent":
            text = self.getParentClassName(curline)
        elif not explicit:
            text = self.getClassName(text)
            
        if not text: 
            if s.autoCActive(): s.autoCCancel()
            return
            
        completions, cancel = self.getClassMembers(text)
        if completions:
            #log.debug("    PHP Completions: %r",completions)
            s.autoCShow(0, completions)
        elif cancel and s.autoCActive():
            s.autoCCancel()
        
    def _VariableAutoComplete(self):
        #log.debug('KoPHPCompletion._VariableAutoComplete')
        # autocomplete works with _entire_ buffer as it scans it for attributes.
        s = self._scintilla
        curpos = s.selectionStart
        curline = s.lineFromPosition(curpos)
        text = self._GetPrevWord()
        #print "PHP text: [%s]"%(text)
        if len(text) < 2 or text=="": 
            if s.autoCActive(): s.autoCCancel()
            return

        completions, cancel = self.getvariables(text, curline)
        #print "PHP Completions: ",repr(completions)
        if completions:
            s.autoCShow(len(text), completions)
        elif cancel and s.autoCActive():
            s.autoCCancel()
    
    def _StartAutoComplete(self):
        #log.debug('KoPHPCompletion._StartAutoComplete')
        # autocomplete works with _entire_ buffer as it scans it for attributes.
        s = self._scintilla
        byte_linelen, lastline = s.getCurLine()
        # fix linelen
        linelen = len(lastline)
        #print "looking for new in [%s]"%(lastline)
        curpos = s.selectionStart
        curline = s.lineFromPosition(curpos)
        bytepos = curpos - s.positionFromLine(curline)
        pos = self.sysUtils.charIndexFromPosition(lastline, bytepos)
        if pos <= 0 or pos > linelen: return
        text = lastline[:pos]
        text = self._GetPrevWordFromText(text)
        #print "PHP text: [%s]"%(text)
        if text == "" and lastline[(pos-1)]==" ":
            pos = pos - 1
            #print "char just typed [%s]"%(lastline[pos])
            text = lastline[:pos]
            text = self._GetPrevWordFromText(text)+" "
        
        if len(text) < 3 or text == "": 
            if s.autoCActive(): s.autoCCancel()
            return
        
        # new class completion
        if text in ["new ", "extends "]:
            completions, cancel = self.getclasses()
            cpos = 0
        elif text == "implements ":
            completions, cancel = self.getinterfaces()
            cpos = 0
        else:
            completions, cancel = self.getmethods(text)
            cpos = len(text)
        if completions:
            #print "PHP Completions: ",completions
            s.autoCShow(cpos, completions)
        elif cancel and s.autoCActive():
            s.autoCCancel()

    def _StartCallTip(self):
        #log.debug('KoPHPCompletion._StartCallTip')
        tip = None
        cname = None
        s = self._scintilla
        byte_linelen, lastline = s.getCurLine()
        linelen = len(lastline)
        curpos = s.selectionStart
        curline = s.lineFromPosition(curpos)
        bytepos = curpos - s.positionFromLine(curline)-1
        pos = self.sysUtils.charIndexFromPosition(lastline, bytepos)
        if pos <= 0: return
        text = lastline[:pos]
        text = self._GetPrevWordFromText(text)

        # we have the function name, is this a class member?
        ctext = lastline[:(pos - len(text))]
        pos = len(ctext)-1
        #print "CALLTIP: ",ctext
        # at least 4 chars required: $a->
        if pos > 2 and ctext[pos]==">" and ctext[pos-1]=="-":
            ctext = lastline[:pos-1]
            ctext = self._GetPrevWordFromText(ctext)
            if ctext == "this":
                cname = self.getCurrentClassName(curline)
            else:
                cname = self.getClassName(ctext)
        # at least 3 chars required for explicit class calls "a::"
        elif pos > 1 and ctext[pos]==":" and ctext[pos-1]==":":
            ctext = lastline[:pos-1]
            #print "CALLTIP text: ",ctext
            cname = self._GetPrevWordFromText(ctext)
            #print "CALLTIP: class ",cname

        if text:
            #print "PHP CallTip on: ", text, " in ",cname
            tip = self.gettipargs(text, cname)
            #print "Call Tip: ",repr(tip)
            if tip:
                self._funcdef = tip 
                self._scintilla.callTipShow(s.positionBefore(curpos), tip)
                self._ContinueCallTip()
        
    def _ContinueCallTip(self):
        #log.debug('KoPHPCompletion._ContinueCallTip')
        s = self._scintilla
        start = 0
        byte_len, current = s.getCurLine()
        l = len(current)
        funcdef = self._funcdef

        if not funcdef: return

        commas = current.count(',')
        if funcdef.find('(') != -1: # XXX hack
            while (start < len(funcdef) and funcdef[start] != '('):
                start = start + 1
        if (start < len(funcdef) and funcdef[start] == '('):
            start = start + 1
        while (start < len(funcdef) and commas > 0) :
            if (funcdef[start] == ',' or funcdef[start] == ')'):
                commas = commas - 1
            start = start + 1
        if (start < len(funcdef) and (funcdef[start] == ',' or funcdef[start] == ')')):
            start = start + 1
        end = start
        if (end < len(funcdef)):
            end = end + 1
        while (end < len(funcdef) and funcdef[end] != ',' and funcdef[end] != ')'):
            end = end + 1
        s.callTipSetHlt(start, end)



    def _GetPrevWord(self, bAllowCalls = 0):
        #log.debug('KoPHPCompletion._GetPrevWord')
        s = self._scintilla
        byte_linelen, lastline = s.getCurLine()
        linelen = len(lastline)
        curpos = s.selectionStart
        linestart = s.positionFromLine(s.lineFromPosition(curpos))
        bytepos = curpos - linestart
        pos = self.sysUtils.charIndexFromPosition(lastline, bytepos)
        fulltext = lastline[:pos]

        index = len(fulltext)-1
        wordbreaks_use = wordbreaks
        if bAllowCalls: wordbreaks_use = wordbreaks_use + "()[]"
        while index>=0:
            char = fulltext[index]
            if char not in wordbreaks_use: break
            index = index-1
        # check preceeding character, dont calltip on variables
        if index >=0:
            char = fulltext[index]
            #if char in variablespec: return ""
        word = fulltext[index+1:]
        return word

    def _GetPrevWordFromText(self, text):
        #log.debug('KoPHPCompletion._GetPrevWordFromText')
        index = len(text)-1
        wordbreaks_use = wordbreaks
        while index>=0:
            char = text[index]
            if char not in wordbreaks_use: break
            index = index-1
        # check preceeding character, dont calltip on variables
        if index >=0:
            char = text[index]
            #if char in variablespec: return ""
        word = text[index+1:]
        return word

