#!python
# Copyright (c) 2000-2006 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

from xpcom import components, ServerException
from koLanguageServiceBase import *
import os
import logging

log = logging.getLogger("koXSLTLanguage")

def registerLanguage(registery):
    registery.registerLanguage(koXSLTLanguage())
    
class koXSLTLanguage(KoLanguageBase):
    name = "XSLT"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{7F76A3CE-7FE1-4363-99EB-4AAAFA79BC88}"

    accessKey = 's'
    primary = 1
    defaultExtension = ".xsl"
    styleBits = 6
    supportsBraceHighlighting = 0
    commentDelimiterInfo = { "block": [ ("<!--", "-->") ],
                             "line": [ ("<!--", "-->") ]}
    namedBlockRE = r'(\<xsl:template.*?\>)'
    namedBlockDescription = 'XSL Templates'

    _block_comment_styles = [sci_constants.SCE_XML_COMMENT_OPEN,
                             sci_constants.SCE_XML_COMMENT_CONTENT,
                             sci_constants.SCE_XML_COMMENT_CLOSE]
    supportsSmartIndent = "XML"
    sample = """<?xml version="1.0"?> 

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="xml" indent="yes"/>

<!-- Syntax Coloring:
    Komodo detects keywords and applies syntax coloring.  In the code
    below, note how "template" is a different color from "match",
    which is a different color from ""Class"". -->

<xsl:template match="Class">
    <html>
            <xsl:apply-templates select="Order"/>
    </html>
</xsl:template>

"""

    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_XSLT)
            self._lexer.supportsFolding = 1
        return self._lexer


    def get_linter(self):
        if self._linter is None:
            self._linter = KoLinterLanguageService("@activestate.com/koXMLLinter;1")
        return self._linter

    def get_completer(self):
        if self._completer is None:
            koDirs = components.classes["@activestate.com/koDirs;1"].\
                                  getService(components.interfaces.koIDirs)
            completionPath = os.path.join(koDirs.mozBinDir, "components" ,"koXSLTCompletion.py")
            if os.path.exists(completionPath):
                self._completer = components.classes["@activestate.com/koXSLTCompletionLanguageService;1"].getService(components.interfaces.koICompletionLanguageService)
            else:
                log.warn('Could not find "%s" for XSLT completion.',
                         completionPath)

        return self._completer

    def getEncodingWarning(self, encoding):
        if not encoding.use_byte_order_marker:
            if encoding.python_encoding_name.startswith('utf-16') or encoding.python_encoding_name.startswith('ucs-'):
                return 'Including a signature (BOM) is recommended for "%s".' % encoding.friendly_encoding_name
            else:
                return ''
        else: # It's all good
            return ''