#!python
# Copyright (c) 2000-2006 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

import os
from xpcom import components
from koLintResult import *
from koLintResults import koLintResults
import re
import process
import sys

class KoXSLTCompileLinter:
    _com_interfaces_ = [components.interfaces.koILinter]
    _reg_desc_ = "Komodo XSLT Compile Linter"
    _reg_clsid_ = "{ae3f7974-78cd-43fc-a06d-99325911e619}"
    _reg_contractid_ = "@activestate.com/koXSLTLinter;1"

    def __init__(self):
        koDirs = components.classes["@activestate.com/koDirs;1"].\
            getService(components.interfaces.koIDirs)

        if sys.platform.startswith("win"):
            _checker = os.path.join(koDirs.supportDir,
                                    'xslt', 'XSLTSyntaxChecker.exe')
        else:
            _checker = os.path.join(koDirs.supportDir,
                                    'xslt', 'XSLTSyntaxChecker')
        _statemachine = os.path.join(koDirs.supportDir, 'xslt', 'state-machine.xml')
        self._argv = [_checker, _statemachine]

        self._resultPattern = re.compile('''Error #: """(.*?)""" \[(\d+)\] @ \((\d+), (\d+)\) => \((\d+), (\d+)\)''')

    def lint(self, request):
        text = request.content.encode(request.encoding.python_encoding_name)
        lintResults = koLintResults()

        p = process.ProcessOpen(self._argv, cwd=request.cwd)
        p.stdin.write(text)
        p.stdin.close()
        results = p.stdout.readlines()
        p.stdout.close()
        p.close()

        for i in results:
            lintResult = KoLintResult()

            lintResult.description, severity,\
                lineStart, columnStart,\
                lineEnd, columnEnd \
                    = self._resultPattern.match(i).groups()                                              

            lintResult.lineStart = int(lineStart)
            lintResult.columnStart = int(columnStart)
            lintResult.lineEnd = int(lineEnd)
            lintResult.columnEnd = int(columnEnd)

            if severity == 1:
                # XML_WARNING
                lintResult.severity = lintResult.SEV_WARNING
            else:
                # XML_ERROR (and call it an error until proven otherwise)
                lintResult.severity = lintResult.SEV_ERROR

            lintResults.addResult( lintResult )
        return lintResults
