# Copyright (c) 2000-2006 ActiveState Software Inc.
# See the file LICENSE.txt for licensing information.

from xpcom import _xpcom, components, Exception
import sys, os
import types

console = None
messageFactory = None

def _checkComponents():
    global console, messageFactory
    if console is None:
        console = components.classes["@mozilla.org/consoleservice;1"].\
            getService(components.interfaces.nsIConsoleService)
        # hook in our listener from koConsole.py
        listener = components.classes["@activestate.com/koConsoleListener;1"].\
            createInstance(components.interfaces.nsIConsoleListener);
        console.registerListener(listener);
        
        # Now see if we can use proxies.
        try:
            console = _xpcom.getProxyForObject(0, 
                                               components.interfaces.nsIConsoleService,
                                               console, _xpcom.PROXY_ASYNC|_xpcom.PROXY_ALWAYS)
        except Exception:
            # assume that the problem is that we're not running inside the GUI (e.g from
            # the Python prompt, and that we don't really _need_ a proxy!
            pass  
    if messageFactory is None:
        messageFactory = components.classes["@activestate.com/koConsoleMessageFactory;1"].\
            getService(components.interfaces.koIConsoleMessageFactory)
        # Surprise #1: The code below hangs Mozilla.
        # Surprise #2: The logger seems to work okay without it.
        # Conclusion: Akuna Matata
        #messageFactory = _xpcom.getProxyForObject(0, 
        #                            components.interfaces.koIConsoleMessageFactory,
        #                            messageFactory, _xpcom.PROXY_SYNC|_xpcom.PROXY_ALWAYS)

def _MakeNiceFilename(fname):
    # Use just the base name, unless it is __init__, in which case
    # also add the parent.
    if not fname:
        return "<unknown!?>"
    path, base = os.path.split(fname)
    if os.path.splitext(base)[0] == "__init__":
        base = os.path.join( os.path.split(path)[1], base)
    return base

_koIsDebugBuildFlag = -1 # -1 == "don't know yet"
# recursion protection, as the info service may itself use logging
_isQueryingInfo = 0

def koIsDebugBuild():
    global _koIsDebugBuildFlag, _isQueryingInfo
    if _koIsDebugBuildFlag==-1 and not _isQueryingInfo:
        # and in some situations (mainly during "bk build") it may fail completely :(
        try:
            _isQueryingInfo = 1
            infoSvc = components.classes["@activestate.com/koInfoService;1"].getService()
            _koIsDebugBuildFlag = infoSvc.buildType == "debug"
        except Exception:
            pass # _koIsDebugBuildFlag remains -1, which will be interpreted as "true"
        _isQueryingInfo = 0
    return _koIsDebugBuildFlag;

def _log(message, category, flag, tag):
    _checkComponents()
    import traceback
    #XXX We should no longer have to go through this to get stack info.
    #    Use the 'instpect' module instead.
    try:
        raise '__'
    except '__', e:
        l = traceback.extract_stack()
        fname, lineNo, funcname, line = l[-3]
        # line is not guaranteed to be a string
        if line == None:
            line = ""
        fname = _MakeNiceFilename(fname)
        line = line + " in '" + funcname + "'"
        m = messageFactory.newMessage()
        if type(message) not in types.StringTypes:
            message = unicode(message)
        m.init(message, fname, line, lineNo, 1, flag, category )  
        if type(message)==type(u''): # may fail auto-string conversion!
            message = repr(message)
        # for debug builds, only write it to one place, as they are both the same anyway!
        if tag != 'Debug' or koIsDebugBuild():
            console.logMessage(m)
            print >>sys.stderr, "*** %s: %s(%d): %s" % (tag, fname, lineNo, message)

def logWarning(message, category="PyXPCOM"):
    _log(message, category,
         components.interfaces.nsIScriptError.warningFlag, "Warning")
def logError(message, category="PyXPCOM"):
    _log(message, category,
         components.interfaces.nsIScriptError.errorFlag, "Error")
def logDebug(message, category="PyXPCOM"):
    _log(message, category,
         components.interfaces.nsIScriptError.warningFlag, "Debug")

